@@ -176,7 +176,7 @@ static GB_TYPE conv_type(const MYSQL_FIELD *f)
 	switch(f->type)
 	{
 		case FIELD_TYPE_TINY:
-			return (f->max_length == 1 ? GB_T_BOOLEAN : GB_T_INTEGER);
+			return (f->max_length == 1 && f->length == 1) ? GB_T_BOOLEAN : GB_T_INTEGER;
 
 		case FIELD_TYPE_INT24:
 		case FIELD_TYPE_SHORT:
@@ -257,7 +257,11 @@ static void conv_string_type(const char *type, MYSQL_FIELD *f)
 		{
 			type += strlen(cst->pattern);
 			if (sscanf(type, "(%ld)", &l) == 1)
+			{
 				f->max_length = l;
+				if (cst->type == FIELD_TYPE_TINY)
+					f->length = l;
+			}
 		}
 	}
 
@@ -292,7 +296,7 @@ static void conv_data(int version, const char *data, long data_length, GB_VARIAN
 	{
 		case FIELD_TYPE_TINY:
 
-			if (f->max_length == 1)
+			if (f->max_length == 1 && f->length == 1)
 			{
 				val->type = GB_T_BOOLEAN;
 				/*GB.NumberFromString(GB_NB_READ_INTEGER, data, strlen(data), &conv);*/
@@ -1246,8 +1250,8 @@ static int begin_transaction(DB_DATABASE *db)
 {
 	/* Autocommit is on by default. Lets set it off. */
 	/* BM: why not doing that when we open the connection ? */
-	do_query(db, "Unable to set autocommit to 0: &1", NULL, "set autocommit=0", 0);
-	return do_query(db, "Unable to begin transaction: &1", NULL, "BEGIN", 0);
+	//do_query(db, "Unable to set autocommit to 0: &1", NULL, "set autocommit=0", 0);
+	return do_query(db, "Unable to begin transaction: &1", NULL, "START TRANSACTION", 0);
 }
 
 
@@ -1269,7 +1273,7 @@ static int commit_transaction(DB_DATABASE *db)
 	bool ret = do_query(db, "Unable to commit transaction: &1", NULL, "COMMIT", 0);
 	/* Autocommit needs to be set back on. */
 	/* BM: and what happens if transactions are imbricated ? */
-	do_query(db, "Unable to set autocommit to On: &1", NULL, "set autocommit=1", 0);
+	//do_query(db, "Unable to set autocommit to On: &1", NULL, "set autocommit=1", 0);
 	return ret;
 }
 
@@ -1298,7 +1302,7 @@ static int rollback_transaction(DB_DATABASE *db)
 	bool ret = do_query(db, "Unable to rollback transaction: &1", NULL, "ROLLBACK", 0);
 	/* Autocommit needs to be set back on. */
 	/* BM: and what happens if transactions are imbricated ? */
-	do_query(db, "Unable to set autocommit to On: &1", NULL, "set autocommit=1", 0);
+	//do_query(db, "Unable to set autocommit to On: &1", NULL, "set autocommit=1", 0);
 	return ret;
 }
 
