@@ -60,7 +60,7 @@ static int _last_error;
 static int check_result(PGresult *res, const char *err)
 {
 	_last_error = 0;
-	
+
 	if (!res)
 	{
 		GB.Error("Out of memory");
@@ -123,7 +123,7 @@ static char *get_quote_string(const char *str, int len, char quote)
 	char *res, *p, c;
 	int len_res;
 	int i;
-	
+
 	len_res = len;
 	for (i = 0; i < len; i++)
 	{
@@ -131,9 +131,9 @@ static char *get_quote_string(const char *str, int len, char quote)
 		if (c == quote || c == '\\' || c == 0)
 			len_res++;
 	}
-	
+
 	res = GB.TempString(NULL, len_res);
-	
+
 	p = res;
 	for (i = 0; i < len; i++)
 	{
@@ -152,7 +152,7 @@ static char *get_quote_string(const char *str, int len, char quote)
 			*p++ = c;
 	}
 	*p = 0;
-	
+
 	return res;
 }
 
@@ -163,13 +163,13 @@ static char *get_quoted_table(const char *table)
 	int len;
 	char *point;
 	char *res;
-	
+
 	if (!table || !*table)
 		return "";
-	
+
 	len = strlen(table);
 	point = index(table, '.');
-	
+
 	if (!point)
 	{
 		res = GB.TempString(NULL, len + 2);
@@ -180,24 +180,24 @@ static char *get_quoted_table(const char *table)
 		res = GB.TempString(NULL, len + 2);
 		sprintf(res, "%.*s.\"%s\"", (int)(point - table), table, point + 1);
 	}
-	
+
 	return res;
 }
 
 static bool get_table_schema(const char **table, char **schema)
 {
 	char *point;
-	
+
 	//fprintf(stderr, "get_table_schema: %s\n", *table);
-	
+
 	*schema = NULL;
-	
+
 	if (!*table || !**table)
 	{
 		//fprintf(stderr, "get_table_schema: -> NULL\n");
 		return TRUE;
 	}
-	
+
 	point = strchr(*table, '.');
 	if (!point)
 	{
@@ -205,7 +205,7 @@ static bool get_table_schema(const char **table, char **schema)
 		*schema = "public";
 		return FALSE;
 	}
-	
+
 	*schema = GB.TempString(*table, point - *table);
 	*table = point + 1;
 	//fprintf(stderr, "get_table_schema: -> %s / %s\n", *schema, *table);
@@ -457,7 +457,7 @@ static void conv_data(const char *data, int len, GB_VARIANT_VALUE *val, Oid type
 				date.year = (-date.year);
 
 			// 4713-01-01 BC is used for null dates
-			
+
 			if (date.year == -4713 && date.month == 1 && date.day == 1)
 				date.year = date.month = date.day = 0;
 
@@ -505,7 +505,7 @@ static void query_get_param(int index, char **str, int *len, char quote)
 	index--;
 	*str = query_param[index];
 	*len = strlen(*str);
-	
+
 	if (quote == '\'')
 	{
 		*str = get_quote_string(*str, *len, quote);
@@ -540,7 +540,7 @@ static int do_query(DB_DATABASE *db, const char *error, PGresult **pres,
 
 	if (DB.IsDebug())
 	{
-		fprintf(stderr, "postgresql: %p: %s\n", conn, query);
+		fprintf(stderr, "gb.db.postgresql: %p: %s\n", conn, query);
 		fflush(stderr);
 	}
 
@@ -593,7 +593,7 @@ static void fill_field_info(DB_DATABASE *db, DB_FIELD *info, PGresult *res, int
 	char *val;
 	Oid type;
 	GB_VARIANT def;
-	
+
 	info->name = NULL;
 
 	type = atoi(PQgetvalue(res, row, col));
@@ -645,7 +645,7 @@ static void fill_field_info(DB_DATABASE *db, DB_FIELD *info, PGresult *res, int
 				GB.StoreVariant(&def, &info->def);
 			}
 		}
-	}	
+	}
 
 	if (db->flags.no_collation)
 		info->collation = NULL;
@@ -704,9 +704,9 @@ static int open_database(DB_DESC *desc, DB_DATABASE *db)
 		GB.Error("Cannot open database: database name too long");
 		return TRUE;
 	}
-	
+
 	//fprintf(stderr, "gb.db.postgresql: host = `%s` port = `%s` dbnname = `%s` user = `%s` password = `%s`\n", desc->host, desc->port, dbname, desc->user, desc->password);
-	
+
 	conn = PQsetdbLogin(desc->host, desc->port, NULL, NULL, dbname, desc->user, desc->password);
 
 	if (!conn)
@@ -737,6 +737,7 @@ static int open_database(DB_DESC *desc, DB_DATABASE *db)
 
 	db->handle = conn;
 	db->version = db_version(db);
+	db->data = (void *)0; // transaction level
 
 	if (db->version >= 90000)
 	{
@@ -751,11 +752,11 @@ static int open_database(DB_DESC *desc, DB_DATABASE *db)
 			return TRUE;
 		}
 	}
-	
+
 	/* flags */
 
 	db->flags.no_table_type = TRUE;
-	db->flags.no_nest = TRUE;
+	//db->flags.no_nest = TRUE;
 	db->flags.no_case = TRUE;
 	db->flags.schema = TRUE;
 	db->flags.no_collation = db->version < 90100;
@@ -1008,7 +1009,7 @@ static void query_release(DB_RESULT result, DB_INFO *info)
 	<next> is a boolean telling if we want the next row.
 
 	This function must return DB_OK, DB_ERROR or DB_NO_DATA
-	
+
 	This function must use GB.StoreVariant() to store the value in the
 	buffer.
 
@@ -1247,7 +1248,19 @@ static int field_length(DB_RESULT result, int field)
 
 static int begin_transaction(DB_DATABASE *db)
 {
-	return do_query(db, "Unable to begin transaction: &1", NULL, "BEGIN", 0);
+	int trans = (int)(intptr_t)(db->data) + 1;
+	db->data = (void *)(intptr_t)trans;
+
+	if (trans == 1)
+	{
+		return do_query(db, "Unable to begin transaction: &1", NULL, "BEGIN", 0);
+	}
+	else
+	{
+		char buffer[8];
+		sprintf(buffer, "%d", trans - 1);
+		return do_query(db, "Unable to begin transaction: &1", NULL, "SAVEPOINT t&1", 1, buffer);
+	}
 }
 
 
@@ -1266,7 +1279,19 @@ static int begin_transaction(DB_DATABASE *db)
 
 static int commit_transaction(DB_DATABASE *db)
 {
-	return do_query(db, "Unable to commit transaction: &1", NULL, "COMMIT", 0);
+	int trans = (int)(intptr_t)(db->data) - 1;
+	db->data = (void *)(intptr_t)trans;
+
+	if (trans == 0)
+	{
+		return do_query(db, "Unable to commit transaction: &1", NULL, "COMMIT", 0);
+	}
+	else
+	{
+		char buffer[8];
+		sprintf(buffer, "%d", trans);
+		return do_query(db, "Unable to begin transaction: &1", NULL, "RELEASE SAVEPOINT t&1", 1, buffer);
+	}
 }
 
 
@@ -1285,7 +1310,19 @@ static int commit_transaction(DB_DATABASE *db)
 
 static int rollback_transaction(DB_DATABASE *db)
 {
-	return do_query(db, "Unable to rollback transaction: &1", NULL, "ROLLBACK", 0);
+	int trans = (int)(intptr_t)(db->data) - 1;
+	db->data = (void *)(intptr_t)trans;
+
+	if (trans == 0)
+	{
+		return do_query(db, "Unable to rollback transaction: &1", NULL, "ROLLBACK", 0);
+	}
+	else
+	{
+		char buffer[8];
+		sprintf(buffer, "%d", trans);
+		return do_query(db, "Unable to begin transaction: &1", NULL, "ROLLBACK TO SAVEPOINT t&1", 1, buffer);
+	}
 }
 
 
@@ -1363,7 +1400,7 @@ static int table_init(DB_DATABASE *db, const char *table, DB_INFO *info)
 							"and pg_attribute.attnum > 0 and not pg_attribute.attisdropped "
 							"and pg_attribute.attrelid = pg_class.oid ";
 	}
-                
+
 	PGresult *res;
 	int i, n;
 	DB_FIELD *f;
@@ -1371,7 +1408,7 @@ static int table_init(DB_DATABASE *db, const char *table, DB_INFO *info)
 
 	// Table name
 	info->table = GB.NewZeroString(table);
-	
+
 	if (get_table_schema(&table, &schema))
 	{
 		if (do_query(db,"Unable to get table fields: &1", &res, qfield_all, 1, table))
@@ -1395,9 +1432,9 @@ static int table_init(DB_DATABASE *db, const char *table, DB_INFO *info)
 	for (i = 0; i < n; i++)
 	{
 		f = &info->field[i];
-		
+
 		fill_field_info(db, f, res, i, 1);
-		
+
 		f->name = GB.NewZeroString(PQgetvalue(res, i, 0));
 	}
 
@@ -1573,7 +1610,7 @@ static int table_exist(DB_DATABASE *db, const char *table)
 	PGresult *res;
 	int exist;
 	char *schema;
-	
+
 	if (get_table_schema(&table, &schema))
 	{
 		if (do_query(db, "Unable to check table: &1", &res, query, 1, table))
@@ -1820,7 +1857,7 @@ static int table_is_system(DB_DATABASE *db, const char *table)
 	char *schema;
 
 	get_table_schema(&table, &schema);
-	
+
 	if (schema)
 		return !strcmp(schema, "pg_catalog");
 
@@ -2176,7 +2213,7 @@ static int field_info(DB_DATABASE *db, const char *table, const char *field, DB_
 	PGresult *res;
 	char *schema;
 	const char *fulltable = table;
-	
+
 	if (get_table_schema(&table, &schema))
 	{
 		if (do_query(db, "Unable to get field info: &1", &res, query, 2, table, field))
@@ -2240,7 +2277,7 @@ static int index_exist(DB_DATABASE *db, const char *table, const char *index)
 	PGresult *res;
 	int exist;
 	char *schema;
-	
+
 	if (get_table_schema(&table, &schema))
 	{
 		if (do_query(db, "Unable to check index: &1", &res, query, 2, table, index))
@@ -2295,7 +2332,7 @@ static int index_list(DB_DATABASE *db, const char *table, char ***indexes)
 	int i;
 	int count;
 	char *schema;
-	
+
 	if (get_table_schema(&table, &schema))
 	{
 		if (do_query(db, "Unable to get indexes: &1", &res, query, 1, table))
