@@ -31,7 +31,9 @@
 
 #ifndef __MAIN_C
 extern GB_INTERFACE GB;
+extern GB_CLASS CLASS_DBusObject;
 extern GB_CLASS CLASS_DBusVariant;
+extern GB_CLASS CLASS_DBusNull;
 #endif
 
 #endif /* __MAIN_H */
