@@ -28,8 +28,10 @@
 
 #include "gambas.h"
 
-GB_INTERFACE GB EXPORT;
+extern GB_INTERFACE GB EXPORT;
 
+#ifndef __C_GSL_C
 extern GB_DESC CGslDesc[];
+#endif
 
 #endif /* __C_GSL_H */
