@@ -27,34 +27,62 @@
 
 #include "main.h"
 
-#ifdef __cplusplus
-extern "C" {
-#endif
+#include "c_gsl.h"
+#include "c_complex.h"
+#include "c_vector.h"
+#include "c_matrix.h"
+#include "c_polynomial.h"
 
 GB_INTERFACE GB EXPORT;
 
-
-GB_CLASS GSL;
-
 GB_DESC *GB_CLASSES[] EXPORT =
 {
   CGslDesc, /* The Elementary math functions */
-  CComplexDesc,
-  CPolynomialDesc,
-  CComplexPolynomialDesc,
+  ComplexDesc,
+	VectorDesc,
+  PolynomialDesc,
+	MatrixDesc,
   /* Other classes go here as completed */
   NULL // Must have a null entry for the end of the structure
 };
 
+GB_CLASS CLASS_Array;
+GB_CLASS CLASS_Complex;
+GB_CLASS CLASS_Matrix;
+GB_CLASS CLASS_Vector;
+GB_CLASS CLASS_Polynomial;
+
+static void error_handler(const char *reason, const char *file, int line, int gsl_errno)
+{
+	//fprintf(stderr, "gb.gsl: error: %s: %s\n", gsl_strerror(gsl_errno), reason);
+	GB.Error("&1: &2", gsl_strerror(gsl_errno), reason);
+}
+
 int EXPORT GB_INIT(void)
 {
+	CLASS_Array = GB.FindClass("Array");
+	CLASS_Complex = GB.FindClass("Complex");
+	CLASS_Vector = GB.FindClass("Vector");
+	CLASS_Matrix = GB.FindClass("Matrix");
+	CLASS_Polynomial = GB.FindClass("Polynomial");
+	
+	gsl_set_error_handler(error_handler);
+	
 	return 0;
 }
 
 void EXPORT GB_EXIT()
 {
+	
 }
 
-#ifdef _cpluscplus
+int EXPORT GB_INFO(const char *key, void **value)
+{
+	if (!strcasecmp(key, "PUSH_COMPLEX"))
+	{
+		*value = (void *)COMPLEX_push_complex;
+		return TRUE;
+	}
+	else
+		return FALSE;
 }
-#endif
