@@ -214,6 +214,7 @@ static void gambas_handle_event(GdkEvent *event)
 	int x, y, xs, ys, xc, yc;
 	bool cancel;
 	int type;
+	bool handle_event = false;
 
 	if (gApplication::_fix_printer_dialog)
 	{
@@ -329,9 +330,6 @@ static void gambas_handle_event(GdkEvent *event)
 			goto __HANDLE_EVENT;
 	}
 
-	//if (event->type == GDK_BUTTON_RELEASE)
-	//	fprintf(stderr, "GDK_BUTTON_RELEASE #2\n");
-
 	if (event->type == GDK_FOCUS_CHANGE)
 	{
 		control = NULL;
@@ -514,6 +512,12 @@ __FOUND_WIDGET:
 		case GDK_BUTTON_RELEASE:
 		{
 			save_control = control = find_child(control, (int)event->button.x_root, (int)event->button.y_root, button_grab);
+			
+			/*if (event->type == GDK_BUTTON_PRESS)
+				fprintf(stderr, "GDK_BUTTON_PRESS: %p / %p\n", control, button_grab);
+			else if (event->type == GDK_BUTTON_RELEASE)
+				fprintf(stderr, "GDK_BUTTON_RELEASE: %p / %p\n", control, button_grab);*/
+			
 			if (!control)
 				goto __HANDLE_EVENT;
 
@@ -782,10 +786,23 @@ __FOUND_WIDGET:
 
 __HANDLE_EVENT:
 
-	gtk_main_do_event(event);
+	handle_event = true;
 
 __RETURN:
 
+	if (event->type == GDK_BUTTON_RELEASE && gApplication::_button_grab)
+	{
+		if (gApplication::_enter_after_button_grab)
+		{
+			gApplication::checkHoveredControl(gApplication::_enter_after_button_grab);
+			gApplication::_enter_after_button_grab = NULL;
+		}
+		gApplication::setButtonGrab(NULL);
+	}
+	
+	if (handle_event)
+		gtk_main_do_event(event);
+
 	if (!gdk_events_pending()) // && event->type != GDK_ENTER_NOTIFY && event->type != GDK_LEAVE_NOTIFY)
 	{
 		if (gApplication::_leave)
@@ -804,16 +821,6 @@ __RETURN:
 		}
 	}
 
-	if (event->type == GDK_BUTTON_RELEASE && gApplication::_button_grab)
-	{
-		if (gApplication::_enter_after_button_grab)
-		{
-			gApplication::checkHoveredControl(gApplication::_enter_after_button_grab);
-			gApplication::_enter_after_button_grab = NULL;
-		}
-		gApplication::setButtonGrab(NULL);
-	}
-
 }
 
 
@@ -1304,7 +1311,7 @@ static void handle_focus_change()
 		return;
 
 	_focus_change = TRUE;
-	GB.Post((void (*)())post_focus_change, NULL);
+	GB.Post((void (*)())post_focus_change, (intptr_t)NULL);
 }
 
 void gApplication::setActiveControl(gControl *control, bool on)
@@ -1364,6 +1371,10 @@ int gApplication::getFrameWidth()
 {
 	int w;
 #ifdef GTK3
+	
+	if (strcmp(getStyleName(), "Breeze") == 0)
+		return 2;
+	
   GtkStyleContext *context = gt_get_style(GTK_TYPE_ENTRY);
   GtkBorder tmp;
 
@@ -1401,7 +1412,13 @@ void gApplication::getBoxFrame(int *pw, int *ph)
 	int w, h;
 
 #ifdef GTK3
-  GtkStyleContext *context = gt_get_style(GTK_TYPE_ENTRY);
+	if (strcmp(getStyleName(), "Breeze") == 0)
+	{
+		*pw = *ph = 2;
+		return;
+	}
+	
+	GtkStyleContext *context = gt_get_style(GTK_TYPE_ENTRY);
   GtkBorder tmp;
 
 	gtk_style_context_get_padding(context, (GtkStateFlags)0, &tmp);
