@@ -22,9 +22,9 @@
 ***************************************************************************/
 
 #include "widgets.h"
-#include "widgets_private.h"
 #include "gmainwindow.h"
 #include "gapplication.h"
+#include "gdesktop.h"
 #include "gmenu.h"
 
 typedef
@@ -36,24 +36,84 @@ typedef
 
 gMenu *gMenu::_current_popup = NULL;
 int gMenu::_in_popup = 0;
+int gMenu::_popup_count = 0;
 
 static GList *menus = NULL;
 
+static gint my_menu_shell_enter_notify(GtkWidget *widget, GdkEventCrossing *event)
+{
+	GtkWidgetClass *klass = GTK_WIDGET_GET_CLASS(widget);
+	GtkWidget *menu_item;
+	gMenu *menu;
+
+	if (event->mode == GDK_CROSSING_GTK_GRAB ||
+			event->mode == GDK_CROSSING_GTK_UNGRAB ||
+			event->mode == GDK_CROSSING_STATE_CHANGED)
+		goto __PREVIOUS;
+
+	menu_item = gtk_get_event_widget((GdkEvent*) event);
+	if (!menu_item)
+		goto __PREVIOUS;
+	
+	menu = (gMenu *)g_object_get_data(G_OBJECT(menu_item), "gambas-menu");
+	if (menu)
+		menu->ensureChildMenu();
+	
+__PREVIOUS:
+
+	return ((gint (*)(GtkWidget *, GdkEventCrossing *))klass->_gtk_reserved6)(widget, event);
+}
+
+static void patch_classes(void)
+{
+	GtkWidgetClass *klass;
+	
+	klass = (GtkWidgetClass *)g_type_class_peek(GTK_TYPE_MENU_SHELL);
+	if (klass->enter_notify_event != my_menu_shell_enter_notify)
+	{
+		//fprintf(stderr, "patch_class: %p\n", klass->enter_notify_event);
+		klass->_gtk_reserved6 = (void (*)())klass->enter_notify_event;
+		klass->enter_notify_event = my_menu_shell_enter_notify;
+	}
+	
+	klass = (GtkWidgetClass *)g_type_class_peek(GTK_TYPE_MENU_BAR);
+	if (klass->enter_notify_event != my_menu_shell_enter_notify)
+	{
+		//fprintf(stderr, "patch_class: %p\n", klass->enter_notify_event);
+		klass->_gtk_reserved6 = (void (*)())klass->enter_notify_event;
+		klass->enter_notify_event = my_menu_shell_enter_notify;
+	}
+}
+
+
 static void mnu_destroy (GtkWidget *object, gMenu *data)
 {
 	if (data->stop_signal) 
 		data->stop_signal = false; 
 	else
-		delete data;
+		data->destroy();
 }
 
 static void mnu_activate(GtkMenuItem *menuitem, gMenu *data)
 {
-	if (!data->child && data->onClick) data->onClick(data);
+	if (data->child)
+		return;
+
+	if (data->radio())
+		data->setRadio();
+	else if (data->toggle())
+		data->setChecked(!data->checked());
+
+	if (data->onClick)
+	{
+		//fprintf(stderr, "mnu_activate: %s\n", data->name());
+		data->onClick(data);
+	}
 }
 
 static gboolean cb_map(GtkWidget *menu, gMenu *data)
 {
+	data->_opened = true;
 	data->hideSeparators();
 	if (data->onShow) (*data->onShow)(data);
 	return false;
@@ -61,24 +121,24 @@ static gboolean cb_map(GtkWidget *menu, gMenu *data)
 
 static gboolean cb_unmap(GtkWidget *menu, gMenu *data)
 {
+	data->_opened = false;
 	if (data->onHide) (*data->onHide)(data);
 	return false;
 }
 
 static int get_menu_pos(GtkWidget *menu)
 {
-  GtkMenuShell *par;
-	GList *iter;
+	GList *children, *iter;
 	int pos;
 	
-	if (!menu->parent)
+	if (!gtk_widget_get_parent(menu))
 	{
 		//g_debug("get_menu_pos: no parent for menu %p", menu);
 		return -1;
 	}
 	
-  par = (GtkMenuShell*)(menu->parent);
-  iter = g_list_first(par->children);
+	children = gtk_container_get_children(GTK_CONTAINER(gtk_widget_get_parent(menu)));
+  iter = g_list_first(children);
   
   for(pos = 0;; pos++) 
   {
@@ -87,51 +147,152 @@ static int get_menu_pos(GtkWidget *menu)
     iter = g_list_next(iter);
   }
   
+  g_list_free(children);
+
   return pos;
 }
 
-static gboolean cb_check_expose(GtkWidget *wid, GdkEventExpose *e, gMenu *menu)
+#ifdef GTK3
+static gboolean cb_check_draw(GtkWidget *wid, cairo_t *cr, gMenu *menu)
 {
-	int x, y, w, h;
-	gint indicator_size;
 	static GtkWidget *check_menu_item = NULL;
-	
-	if (menu->checked())
-	{
-		x = wid->allocation.x;
-		y = wid->allocation.y;
-		w = wid->allocation.width;
-		h = wid->allocation.height;
-		
-		indicator_size = 15;
-		if (w < indicator_size) indicator_size = w;
-		if (h < indicator_size) indicator_size = h;
-		
-		x += (w - indicator_size) / 2;
-		y += (h - indicator_size) / 2;
-		w = indicator_size;
-		h = indicator_size;
+	static GtkWidget *radio_menu_item = NULL;
 
+	int x, y, w, h;
+	gint indicator_size = 0;
+	GtkStateFlags state;
+	GtkWidget *item;
+	GtkStyleContext *style;
+
+	GtkAllocation a;
+	gtk_widget_get_allocation(wid, &a);
+	x = 0; //a.x;
+	y = 0; //a.y;
+	w = a.width;
+	h = a.height;
+
+	if (menu->radio())
+	{
+		if (!radio_menu_item)
+			radio_menu_item = gtk_radio_menu_item_new(NULL);
+		item = radio_menu_item;
+		style = gtk_widget_get_style_context(item);
+		gtk_style_context_add_class(style, GTK_STYLE_CLASS_RADIO);
+	}
+	else
+	{
 		if (!check_menu_item)
 			check_menu_item = gtk_check_menu_item_new();
-		
-		gtk_widget_set_state(check_menu_item, (GtkStateType)GTK_WIDGET_STATE(wid));
-		
+		item = check_menu_item;
+		style = gtk_widget_get_style_context(item);
+		gtk_style_context_add_class(style, GTK_STYLE_CLASS_CHECK);
+	}
+
+	gtk_widget_style_get(item, "indicator-size", &indicator_size, (char *)NULL);
+	indicator_size = MAX(16, indicator_size);
+
+	x += (w - indicator_size) / 2;
+	y += (h - indicator_size) / 2;
+	w = indicator_size;
+	h = indicator_size;
+
+	state = gtk_widget_get_state_flags(wid);
+
+	if (menu->checked())
+		state = (GtkStateFlags)((int)state | GTK_STATE_FLAG_ACTIVE
+#if GTK_CHECK_VERSION(3, 14, 0)
+		| GTK_STATE_FLAG_CHECKED
+#endif
+		);
+
+	gtk_widget_set_state_flags(item, state, true);
+
+	style = gtk_widget_get_style_context(wid);
+	gtk_style_context_save(style);
+	gtk_style_context_set_state(style, state);
+
+	if (menu->radio())
+	{
+    gtk_style_context_add_class(style, GTK_STYLE_CLASS_RADIO);
+		//gtk_render_background(style, cr, x, y, w, h);
+		gtk_render_option(style, cr, x, y, indicator_size, indicator_size);
+	}
+	else
+	{
+    gtk_style_context_add_class(style, GTK_STYLE_CLASS_CHECK);
+		//gtk_render_background(style, cr, x, y, w, h);
+		gtk_render_check(style, cr, x, y, indicator_size, indicator_size);
+	}
+
+	gtk_style_context_restore(style);
+	return false;
+}
+#else
+static gboolean cb_check_expose(GtkWidget *wid, GdkEventExpose *e, gMenu *menu)
+{
+	static GtkWidget *check_menu_item = NULL;
+	static GtkWidget *radio_menu_item = NULL;
+	GtkWidget *item;
+
+	int x, y, w, h;
+	gint indicator_size;
+	GtkShadowType shadow;
+
+	x = wid->allocation.x;
+	y = wid->allocation.y;
+	w = wid->allocation.width;
+	h = wid->allocation.height;
+
+	if (menu->radio())
+	{
+		if (!radio_menu_item)
+			radio_menu_item = gtk_radio_menu_item_new(NULL);
+		item = radio_menu_item;
+	}
+	else
+	{
+		if (!check_menu_item)
+			check_menu_item = gtk_check_menu_item_new();
+		item = check_menu_item;
+	}
+
+	gtk_widget_style_get(item, "indicator-size", &indicator_size, (char *)NULL);
+	indicator_size = MAX(16, indicator_size);
+
+	x += (w - indicator_size) / 2;
+	y += (h - indicator_size) / 2;
+	w = indicator_size;
+	h = indicator_size;
+
+	gtk_widget_set_state(item, (GtkStateType)GTK_WIDGET_STATE(wid));
+
+	shadow = menu->checked() ? GTK_SHADOW_IN : GTK_SHADOW_OUT;
+
+	if (menu->radio())
+	{
+		gtk_paint_option(wid->style, wid->window,
+					(GtkStateType)GTK_WIDGET_STATE(wid), shadow,
+					&e->area, radio_menu_item, "radiobutton",
+					x, y, w, h);
+	}
+	else
+	{
+
 		gtk_paint_check(wid->style, wid->window,
-					(GtkStateType)GTK_WIDGET_STATE(wid), GTK_SHADOW_IN,
+					(GtkStateType)GTK_WIDGET_STATE(wid), shadow,
 					&e->area, check_menu_item, "check",
-					x + 1, y + 1, w - 2, h - 2);
-		
+					x, y, w, h);
 	}
 	
 	return false;
 }
+#endif
 
 void gMenu::update()
 {
 	GtkMenuShell *shell = NULL;
 	gint pos;
-	GtkRequisition req;
+	int size;
 	
 	if (_no_update)
 		return;
@@ -144,7 +305,7 @@ void gMenu::update()
 		{
 			g_object_ref(G_OBJECT(child));
 			if (_style == MENU)
-				gtk_menu_item_remove_submenu(menu);
+				gtk_menu_item_set_submenu(menu, NULL);
 		}
 
 		if (menu)
@@ -167,9 +328,13 @@ void gMenu::update()
 			if (_style == SEPARATOR)
 			{
 				menu = (GtkMenuItem *)gtk_separator_menu_item_new();
+#ifdef GTK3
+#else
+				GtkRequisition req;
 				gtk_widget_size_request(GTK_WIDGET(menu), &req);
 				if (req.height > 5)
 					gtk_widget_set_size_request(GTK_WIDGET(menu), -1, 5);
+#endif
 				//g_debug("%p: create new separator %p", this, menu);
 			}
 			else if (_style == MENU)
@@ -177,7 +342,7 @@ void gMenu::update()
 				menu = (GtkMenuItem *)gtk_image_menu_item_new();
 				//g_debug("%p: create new menu %p", this, menu);
 				
-				hbox = gtk_hbox_new(false, 4);
+				hbox = gtk_hbox_new(false, gDesktop::scale());
 				//set_gdk_bg_color(hbox, 0xFF0000);
 				gtk_container_add(GTK_CONTAINER(menu), GTK_WIDGET(hbox));
 				
@@ -195,10 +360,11 @@ void gMenu::update()
 					
 					check = gtk_image_new();
 					g_object_ref(check);
-					gtk_widget_set_size_request(check, 16, 16);
-					g_signal_connect_after(G_OBJECT(check), "expose-event", G_CALLBACK(cb_check_expose), (gpointer)this);
+					size = MAX(18, window()->font()->height());
+					gtk_widget_set_size_request(check, size, size);
+					ON_DRAW(check, this, cb_check_expose, cb_check_draw);
+					//g_signal_connect_after(G_OBJECT(check), "expose-event", G_CALLBACK(cb_check_expose), (gpointer)this);
 					
-					//gtk_box_pack_start(GTK_BOX(hbox), check, false, false, 0);
 					//gtk_box_pack_start(GTK_BOX(hbox), image, false, false, 0);
 					gtk_box_pack_start(GTK_BOX(hbox), label, false, false, 0);
 					gtk_box_pack_end(GTK_BOX(hbox), aclbl, false, false, 0);				
@@ -211,7 +377,7 @@ void gMenu::update()
 				
 				if (child)
 				{
-					gtk_menu_item_set_submenu(GTK_MENU_ITEM(menu), GTK_WIDGET(child));
+					gtk_menu_item_set_submenu(menu, GTK_WIDGET(child));
 					g_object_unref(G_OBJECT(child));
 				}
 					
@@ -239,13 +405,14 @@ void gMenu::update()
 				if (!parent->child)
 				{
 					parent->child = (GtkMenu*)gtk_menu_new();
+					
 					//g_debug("%p: creates a new child menu container in parent %p", this, parent->child);
 					
 					g_signal_connect(G_OBJECT(parent->child), "map", G_CALLBACK(cb_map), (gpointer)parent);
 					g_signal_connect(G_OBJECT(parent->child), "unmap", G_CALLBACK(cb_unmap), (gpointer)parent);
 					gtk_widget_show_all(GTK_WIDGET(parent->child));
 					if (parent->style() == MENU)
-						gtk_menu_item_set_submenu(GTK_MENU_ITEM(parent->menu), GTK_WIDGET(parent->child));
+						gtk_menu_item_set_submenu(parent->menu, GTK_WIDGET(parent->child));
 					
 					parent->setColor();
 					
@@ -257,6 +424,8 @@ void gMenu::update()
 	
 			if (shell)
 			{
+				patch_classes();
+				
 				if (pos < 0)
 				{
 					gtk_menu_shell_append(shell, GTK_WIDGET(menu));
@@ -271,6 +440,8 @@ void gMenu::update()
 			
 			g_signal_connect(G_OBJECT(menu), "destroy", G_CALLBACK(mnu_destroy), (gpointer)this);
 			g_signal_connect(G_OBJECT(menu), "activate", G_CALLBACK(mnu_activate), (gpointer)this);
+			
+			g_object_set_data(G_OBJECT(menu), "gambas-menu", this);
 		}
 		
 		_oldstyle = _style;
@@ -298,39 +469,20 @@ void gMenu::update()
 			}
 			else
 				gtk_label_set_text(GTK_LABEL(aclbl), "\t");
-		}
-		
-		if (!top_level)
-		{
-			if (!_picture && !_checked)
-			{
-				gtk_image_set_from_pixbuf(GTK_IMAGE(image), NULL);
-				gtk_image_menu_item_set_image((GtkImageMenuItem *)menu, image);
-			}
-			else if (_checked)
+
+			if (_checked || _radio || _toggle)
 			{
 				gtk_image_menu_item_set_image((GtkImageMenuItem *)menu, check);
+				gtk_widget_show(check);
+				gtk_widget_hide(image);
 			}
 			else
 			{
 				gtk_image_set_from_pixbuf(GTK_IMAGE(image), _picture ? _picture->getPixbuf() : NULL);
 				gtk_image_menu_item_set_image((GtkImageMenuItem *)menu, image);
-			}
-			
-			/*if (buf)
 				gtk_widget_show(image);
-			else
-				gtk_widget_hide(image);*/
-				
-			/*if (_checked || !buf)
-				gtk_widget_show(check);
-			else
-				gtk_widget_hide(check);*/
-		}
-		else
-		{
-			//gtk_widget_hide(image);
-			//gtk_widget_hide(check);			
+				gtk_widget_hide(check);
+			}
 		}
 		
 		setColor();
@@ -366,18 +518,23 @@ void gMenu::initialize()
 	_picture = NULL;
 	_name = NULL;
 	_toggle = false;
-	
+	_radio = false;
+
 	_style = NOTHING;
 	_oldstyle = NOTHING;
 	
 	_no_update = false;
 	_destroyed = false;
+	_delete_later = false;
 	_action = false;
 	_visible = false;
+	_opened = false;
+	
+	_proxy = NULL;
 	
 	sizeGroup = gtk_size_group_new(GTK_SIZE_GROUP_HORIZONTAL);
 	
-	menus=g_list_append (menus,(gpointer)this);
+	menus = g_list_append (menus,(gpointer)this);
 }
 
 
@@ -413,7 +570,7 @@ gMenu::gMenu(gMenu *par, bool hidden)
 	pr = (gpointer)par;
 	
 	initialize();
-	sizeGroup=gtk_size_group_new(GTK_SIZE_GROUP_HORIZONTAL);
+	//sizeGroup = gtk_size_group_new(GTK_SIZE_GROUP_HORIZONTAL);
 	
 	if (!par) return;
 	if (!par->menu) return;
@@ -437,14 +594,18 @@ gMenu::~gMenu()
 	
 	_destroyed = true;
   
+	setProxy(NULL);
+	
   // Remove references to me
   
+#define CLEAR_POINTER(_field) if ((_field) == (void *)this) _field = NULL
+	
 	item = g_list_first(menus);
 	while (item)
 	{
 		mn = (gMenu*)item->data;
-		if (mn->pr == (void*)this)
-		  mn->pr = NULL;
+		CLEAR_POINTER(mn->pr);
+		CLEAR_POINTER(mn->_proxy);
 		item = g_list_next(item);
 	}
 	
@@ -460,9 +621,6 @@ gMenu::~gMenu()
 		if (aclbl && (!top_level) && pr)
 			gtk_size_group_remove_widget(((gMenu*)pr)->sizeGroup, aclbl);
 		
-		if (child)
-			gtk_widget_destroy(GTK_WIDGET(child));
-	
 		if (sizeGroup) 
 			g_object_unref(G_OBJECT(sizeGroup));
 		
@@ -472,6 +630,9 @@ gMenu::~gMenu()
 		
 	_style = NOTHING;
 	
+	if (child)
+		gtk_widget_destroy(GTK_WIDGET(child));
+	
 	stop_signal = true;
 	
 	if (image)
@@ -491,7 +652,7 @@ gMenu::~gMenu()
 
 bool gMenu::enabled()
 {
-	return GTK_WIDGET_SENSITIVE(GTK_WIDGET(menu));
+	return gtk_widget_is_sensitive(GTK_WIDGET(menu));
 }
 
 void gMenu::setEnabled(bool vl)
@@ -563,6 +724,13 @@ void gMenu::setChecked(bool vl)
 void gMenu::setToggle(bool vl)
 {
 	_toggle = vl;
+	update();
+}
+
+void gMenu::setRadio(bool vl)
+{
+	_radio = vl;
+	update();
 }
 
 int gMenu::childCount()
@@ -570,17 +738,18 @@ int gMenu::childCount()
 	GList *item;
 	gMenu *mn;
 	int ct=0;
-
+	
 	if (!menus) return 0;
-
+	
 	item=g_list_first(menus);
 	while (item)
 	{
 		mn=(gMenu*)item->data;
-		if (mn->pr == (void*)this) ct++;
+		if (mn->pr == (void*)this && !mn->_delete_later)
+			ct++;
 		item=g_list_next(item);
 	}
-
+	
 	return ct;
 }
 
@@ -589,27 +758,27 @@ gMenu* gMenu::childMenu(int pos)
 	GList *item;
 	gMenu *mn;
 	int ct=0;
-
+	
 	if (!menus) return NULL;
-
+	
 	item=g_list_first(menus);
 	while (item)
 	{
 		mn=(gMenu*)item->data;
-		if (mn->pr == (void*)this)
+		if (mn->pr == (void*)this && !mn->_delete_later)
 		{
 			if (ct==pos) return mn;
 			ct++;
 		}
 		item=g_list_next(item);
 	}
-
+	
 	return NULL;
 }
 
 void gMenu::destroy()
 {
-	if (!_destroyed)
+	if (!_destroyed && !_delete_later)
 		delete this;
 }
 
@@ -640,10 +809,15 @@ void gMenu::doPopup(bool move, int x, int y)
 	_current_popup = this;
 	
 	_in_popup++;
-
+	_popup_count++;
+	
 	gtk_menu_popup(child, NULL, NULL, move ? (GtkMenuPositionFunc)position_menu : NULL, (gpointer)pos, 0, gApplication::lastEventTime());
 	
+#if GTK_CHECK_VERSION(2, 20, 0)
+	while (_current_popup && child && gtk_widget_get_mapped(GTK_WIDGET(child)))
+#else
 	while (_current_popup && child && GTK_WIDGET_MAPPED(child))
+#endif
 		MAIN_do_iteration(false);
 
 	_current_popup = save_current_popup;
@@ -666,6 +840,13 @@ void gMenu::popup()
 	doPopup(false);
 }
 
+void gMenu::close()
+{
+	if (!child)
+		return;
+	
+	gtk_menu_popdown(child);
+}
 
 int gMenu::winChildCount(gMainWindow *par)
 {
@@ -811,7 +992,10 @@ void gMenu::hideSeparators()
 			else
 			{
 				if (ch->isVisible())
+				{
+					ch->ensureChildMenu();
 					last_sep = false;
+				}
 			}
 		}
 		
@@ -825,8 +1009,13 @@ void gMenu::hideSeparators()
 void gMenu::setFont()
 {
 	gMainWindow *win = window();
-	if (label) gtk_widget_modify_font(GTK_WIDGET(label), win->ownFont() ? win->font()->desc() : NULL);
-	if (aclbl) gtk_widget_modify_font(GTK_WIDGET(aclbl), win->ownFont() ? win->font()->desc() : NULL);
+#ifdef GTK3
+	if (label) gtk_widget_override_font(GTK_WIDGET(label), win->font()->desc());
+	if (aclbl) gtk_widget_override_font(GTK_WIDGET(aclbl), win->font()->desc());
+#else
+	if (label) gtk_widget_modify_font(GTK_WIDGET(label), win->font()->desc());
+	if (aclbl) gtk_widget_modify_font(GTK_WIDGET(aclbl), win->font()->desc());
+#endif
 }
 
 void gMenu::setColor()
@@ -872,7 +1061,11 @@ void gMenu::updateFont(gMainWindow *win)
 	if (win->menuBar)
 	{
 		//fprintf(stderr, "set menu bar font\n");
-		gtk_widget_modify_font(GTK_WIDGET(win->menuBar), win->ownFont() ? win->font()->desc() : NULL);	
+#ifdef GTK3
+		gtk_widget_override_font(GTK_WIDGET(win->menuBar), win->ownFont() ? win->font()->desc() : NULL);
+#else
+		gtk_widget_modify_font(GTK_WIDGET(win->menuBar), win->ownFont() ? win->font()->desc() : NULL);
+#endif
 	}
 	
 	if (!menus) 
@@ -888,17 +1081,79 @@ void gMenu::updateFont(gMainWindow *win)
 	}
 }
 
-/*void gMenu::setTearOff(bool v)
+void gMenu::setRadio()
 {
-	if (child)
-		gtk_menu_set_tearoff_state(child, v);
+	gMenu *child;
+	GList *item;
+	GList *start = NULL;
+
+	if (top_level)
+		return;
+
+	item = g_list_first(menus);
+	while (item)
+	{
+		child = (gMenu*)item->data;
+		if (child->parent() == pr && !child->_delete_later)
+		{
+			if (child->radio())
+			{
+				if (!start)
+					start = item;
+				if (child == this)
+					break;
+			}
+			else
+				start = NULL;
+		}
+
+		item = g_list_next(item);
+	}
+
+	item = start;
+	while (item)
+	{
+		child = (gMenu*)item->data;
+		if (child->parent() == pr && !child->_delete_later)
+		{
+			if (!child->radio())
+				break;
+
+			child->setChecked(child == this);
+		}
+
+		item = g_list_next(item);
+	}
 }
 
-bool gMenu::isTearOff() const
+void gMenu::setProxy(gMenu *menu)
 {
+	_proxy = menu;
+}
+
+GtkMenu *gMenu::getSubMenu()
+{
+	if (_proxy)
+		return _proxy->getSubMenu();
+	else
+		return child;
+}
+
+void gMenu::ensureChildMenu()
+{
+	GtkWidget *child = GTK_WIDGET(getSubMenu());
+	
 	if (child)
-		return gtk_menu_get_tearoff_state(child);
-	else
-		return false;
-}*/
+	{
+		// TODO: create parent menu?
+		if (gtk_menu_item_get_submenu(menu) != child)
+		{
+			//fprintf(stderr, "ensureChildMenu: %s\n", name());
+			g_object_ref(child);
+			gtk_menu_item_set_submenu(GTK_MENU_ITEM(gtk_menu_get_attach_widget(GTK_MENU(child))), NULL);
+			gtk_menu_item_set_submenu(menu, child);
+			g_object_unref(child);
+		}
+	}
+}
 
