@@ -58,6 +58,7 @@ typedef
 		void (*get_preferred_width_for_height)(GtkWidget *widget, gint height, gint *minimum_width, gint *natural_width);
 		void (*get_preferred_width)(GtkWidget *widget, gint *minimum_width, gint *natural_width);
 		void (*get_preferred_height_for_width)(GtkWidget *widget, gint width, gint *minimum_height, gint *natural_height);
+		//void (*get_preferred_height_and_baseline_for_width)(GtkWidget *widget, gint width, gint *minimum, gint *natural, gint *minimum_baseline, gint *natural_baseline);
 	}
 	PATCH_FUNCS;
 
@@ -329,6 +330,7 @@ void gControl::initAll(gContainer *parent)
 
 	_fg = _bg = COLOR_DEFAULT;
 #ifdef GTK3
+	_css = NULL;
 	_fg_name = _bg_name = NULL;
 #endif
 
@@ -351,6 +353,9 @@ gControl::~gControl()
 
 	emit(SIGNAL(onFinish));
 
+	/*if (pr)
+		pr->remove(this);*/
+	
 	if (win && win->focus == this)
 		win->focus = NULL;
 
@@ -374,6 +379,11 @@ gControl::~gControl()
 		gFont::assign(&_resolved_font);
 	}
 
+#ifdef GTK3
+	if (_css)
+		g_object_unref(_css);
+#endif
+	
 	//fprintf(stderr, "~gControl: %s\n", name());
 
 	setName(NULL);
@@ -1554,8 +1564,34 @@ static void type##_get_preferred_height_for_width(GtkWidget *widget, gint width,
 	} \
 	GtkWidgetClass *klass = (GtkWidgetClass *)g_type_class_peek(type); \
 	(*OLD_FUNC->get_preferred_height_for_width)(widget, width, minimum_size, natural_size); \
+} \
+static void type##_get_preferred_width_for_height(GtkWidget *widget, gint height, gint *minimum_size, gint *natural_size) \
+{ \
+	if (minimum_size && must_patch(widget)) \
+	{ \
+		*minimum_size = 0; \
+		*natural_size = 0; \
+		return; \
+	} \
+	GtkWidgetClass *klass = (GtkWidgetClass *)g_type_class_peek(type); \
+	(*OLD_FUNC->get_preferred_height_for_width)(widget, height, minimum_size, natural_size); \
 }
 
+#define PATCH_DECLARE_BASELINE(type) \
+static void type##get_preferred_height_and_baseline_for_width(GtkWidget *widget, gint width, gint *minimum, gint *natural, gint *minimum_baseline, gint *natural_baseline) \
+{ \
+	if (minimum && minimum_baseline && must_patch(widget)) \
+	{ \
+		*minimum = 0; \
+		*natural = 0; \
+		*minimum_baseline = 0; \
+		*natural_baseline = 0; \
+		fprintf(stderr, "%s\n", __func__); \
+		return; \
+	} \
+	GtkWidgetClass *klass = (GtkWidgetClass *)g_type_class_peek(type); \
+	(*OLD_FUNC->get_preferred_height_and_baseline_for_width)(widget, width, minimum, natural, minimum_baseline, natural_baseline); \
+}
 
 //fprintf(stderr, "patching [%p %s] (%p %p)\n", klass, G_OBJECT_TYPE_NAME(widget), klass->get_preferred_width, klass->get_preferred_height);
 //		fprintf(stderr, "PATCH_CLASS: %s\n", G_OBJECT_TYPE_NAME(widget));
@@ -1575,11 +1611,33 @@ if (G_OBJECT_TYPE(widget) == type) \
 		klass->get_preferred_width = type##_get_preferred_width; \
 		klass->get_preferred_height = type##_get_preferred_height; \
 		klass->get_preferred_height_for_width = type##_get_preferred_height_for_width; \
+		klass->get_preferred_width_for_height = type##_get_preferred_width_for_height; \
+	} \
+}
+
+#define PATCH_CLASS_BASELINE(widget, type) \
+if (G_OBJECT_TYPE(widget) == type) \
+{ \
+	GtkWidgetClass *klass = (GtkWidgetClass *)GTK_WIDGET_GET_CLASS(widget); \
+	if (klass->get_preferred_width != type##_get_preferred_width) \
+	{ \
+		PATCH_FUNCS *funcs = g_new0(PATCH_FUNCS, 1); \
+		funcs->get_preferred_width = klass->get_preferred_width; \
+		funcs->get_preferred_height = klass->get_preferred_height; \
+		funcs->get_preferred_height_for_width = klass->get_preferred_height_for_width; \
+		funcs->get_preferred_width_for_height = klass->get_preferred_width_for_height; \
+		funcs->get_preferred_height_and_baseline_for_width = klass->get_preferred_height_and_baseline_for_width; \
+		klass->_gtk_reserved6 = (void(*)())funcs; \
+		klass->get_preferred_width = type##_get_preferred_width; \
+		klass->get_preferred_height = type##_get_preferred_height; \
+		klass->get_preferred_height_for_width = type##_get_preferred_height_for_width; \
+		klass->get_preferred_height_and_baseline_for_width = type##get_preferred_height_and_baseline_for_width; \
 	} \
 }
 
 PATCH_DECLARE(GTK_TYPE_WINDOW)
 PATCH_DECLARE(GTK_TYPE_ENTRY)
+//PATCH_DECLARE_BASELINE(GTK_TYPE_ENTRY)
 PATCH_DECLARE(GTK_TYPE_COMBO_BOX)
 PATCH_DECLARE(GTK_TYPE_SPIN_BUTTON)
 PATCH_DECLARE(GTK_TYPE_BUTTON)
@@ -1644,6 +1702,7 @@ void gControl::realize(bool make_frame)
 
 	PATCH_CLASS(border, GTK_TYPE_WINDOW)
 	else PATCH_CLASS(border, GTK_TYPE_ENTRY)
+	//else PATCH_CLASS_BASELINE(border, GTK_TYPE_ENTRY)
 	else PATCH_CLASS(border, GTK_TYPE_SPIN_BUTTON)
 	else PATCH_CLASS(border, GTK_TYPE_BUTTON)
 	else PATCH_CLASS(border, GTK_TYPE_FIXED)
@@ -1828,6 +1887,71 @@ void gControl::setName(char *name)
 
 #ifdef GTK3
 
+GtkWidget *gControl::getStyleSheetWidget()
+{
+	return border;
+}
+
+void gControl::updateStyleSheet()
+{
+	static int count = 0;
+	
+	GtkWidget *wid;
+	GtkStyleContext *context;
+	char *css = NULL;
+	const char *name;
+	char buffer[128];
+	
+	wid = getStyleSheetWidget();
+	context = gtk_widget_get_style_context(wid);
+	
+	if (_bg == COLOR_DEFAULT && _fg == COLOR_DEFAULT)
+	{
+		if (_css)
+			gtk_style_context_remove_provider(context, _css);
+	}
+	else
+	{
+		if (!_css)
+		{
+			count++;
+			sprintf(buffer, "g%d", count);
+			gtk_widget_set_name(wid, buffer);
+			
+			_css = GTK_STYLE_PROVIDER(gtk_css_provider_new());
+		}
+		else
+			gtk_style_context_remove_provider(context, _css);
+		
+		name = gtk_widget_get_name(wid);
+		sprintf(buffer, "#%s:not(:selected) {\n", name);
+		g_stradd(&css, buffer);
+		
+		if (_bg != COLOR_DEFAULT)
+		{
+			g_stradd(&css, "background-color:");
+			gt_to_css_color(buffer, _bg);
+			g_stradd(&css, buffer);
+			g_stradd(&css, ";\nbackground-image:none;\n");
+		}
+		
+		if (_fg != COLOR_DEFAULT)
+		{
+			g_stradd(&css, "color:");
+			gt_to_css_color(buffer, _fg);
+			g_stradd(&css, buffer);
+			g_stradd(&css, ";\n");
+		}
+		
+		g_stradd(&css, "}\n");
+		
+		//fprintf(stderr, "---- %s\n%s", _name, css);
+		
+		gtk_css_provider_load_from_data(GTK_CSS_PROVIDER(_css), css, -1, NULL);
+		gtk_style_context_add_provider(context, _css, GTK_STYLE_PROVIDER_PRIORITY_USER);
+	}
+}
+
 gColor gControl::realBackground(bool no_default)
 {
 	if (_bg != COLOR_DEFAULT)
@@ -1850,7 +1974,8 @@ void gControl::setRealBackground(gColor color)
 void gControl::setBackground(gColor color)
 {
 	_bg = color;
-	gt_widget_set_color(border, FALSE, _bg, _bg_name, &_bg_default);
+	updateStyleSheet();
+	//gt_widget_set_color(border, FALSE, _bg, _bg_name, &_bg_default);
 	updateColor();
 }
 
@@ -1876,7 +2001,8 @@ void gControl::setRealForeground(gColor color)
 void gControl::setForeground(gColor color)
 {
 	_fg = color;
-	gt_widget_set_color(border, TRUE, _fg, _fg_name, &_fg_default);
+	updateStyleSheet();
+	//gt_widget_set_color(border, TRUE, _fg, _fg_name, &_fg_default);
 	updateColor();
 }
 
@@ -2008,14 +2134,14 @@ void gControl::reparent(gContainer *newpr, int x, int y)
 
 	if (oldpr == newpr)
 	{
-		gtk_widget_reparent(border, newpr->getContainer());
+		gt_widget_reparent(border, newpr->getContainer());
 		oldpr->performArrange();
 	}
 	else
 	{
 		if (oldpr)
 		{
-			gtk_widget_reparent(border, newpr->getContainer());
+			gt_widget_reparent(border, newpr->getContainer());
 			oldpr->remove(this);
 			oldpr->performArrange();
 		}
