@@ -61,19 +61,27 @@ static int get_length(void *_object)
 
 static void to_pos(QTextEdit *wid, int par, int car, int *pos)
 {
-	QTextCursor cursor = wid->textCursor();
-	QTextBlock block = cursor.block();
+	QTextCursor cursor;
+	QTextBlock block;
 	int p = 0;
 
+	cursor = wid->textCursor();
+	cursor.movePosition(QTextCursor::Start);
+	
+	block = cursor.block();
+	
 	while (par)
 	{
 		if (!block.isValid())
 			break;
-		p += block.length() + 1;
+		p += block.length();
 		block = block.next();
 		par--;
 	}
 
+	if (block.isValid())
+		car = qMin(block.length() - 1, car);
+	
 	*pos = p + car;
 }
 
@@ -574,17 +582,19 @@ BEGIN_PROPERTY(TextArea_Border)
 	
 END_PROPERTY
 
-/*
-GB_DESC CTextAreaLinesDesc[] =
-{
-	GB_DECLARE(".TextArea.Lines", 0), GB_VIRTUAL_CLASS(),
+BEGIN_METHOD(TextArea_CursorAt, GB_INTEGER pos)
 
-	GB_METHOD("_get", "i", CTEXTAREA_line_get, "s"),
-	GB_PROPERTY_READ("Count", "i", CTEXTAREA_line_count),
+	QRect rect;
+	QTextCursor cursor = WIDGET->textCursor();
+	
+	if (!MISSING(pos))
+		cursor.setPosition(VARG(pos));
+	
+	rect = WIDGET->cursorRect(cursor);
+	
+	GB.ReturnObject(GEOM.CreatePoint(rect.x() + WIDGET->viewport()->x(), rect.bottom() + WIDGET->viewport()->y()));
 
-	GB_END_DECLARE
-};
-*/
+END_PROPERTY
 
 GB_DESC CTextAreaSelectionDesc[] =
 {
@@ -641,6 +651,8 @@ GB_DESC CTextAreaDesc[] =
 	GB_METHOD("ToLine", "i", CTEXTAREA_to_line, "(Pos)i"),
 	GB_METHOD("ToColumn", "i", CTEXTAREA_to_col, "(Pos)i"),
 
+	GB_METHOD("CursorAt", "Point", TextArea_CursorAt, "[(Pos)i]"),
+
 	GB_METHOD("EnsureVisible", NULL, CTEXTAREA_ensure_visible, NULL),
 
 	GB_EVENT("Change", NULL, NULL, &EVENT_Change),
