@@ -119,7 +119,7 @@ extern "C" {
 
 const GB_INTERFACE *GB_PTR EXPORT;
 IMAGE_INTERFACE IMAGE EXPORT;
-
+GEOM_INTERFACE GEOM EXPORT;
 }
 
 int MAIN_in_wait = 0;
@@ -897,6 +897,15 @@ static void QT_Init(void)
 			qApp->setStyle(new FixBreezeStyle);
 		}
 	}
+	else if (::strcmp(qApp->style()->metaObject()->className(), "Oxygen::Style") == 0)
+	{
+		char *env = getenv("GB_QT_NO_OXYGEN_FIX");
+		if (!env || atoi(env) == 0)
+		{
+			CSTYLE_fix_breeze = TRUE;
+			qApp->setStyle(new FixBreezeStyle);
+		}
+	}
 
 
 	MAIN_update_scale(qApp->desktop()->font());
@@ -1387,8 +1396,12 @@ int EXPORT GB_INIT(void)
 	GB.Component.Load("gb.draw");
 	GB.Component.Load("gb.image");
 	GB.Component.Load("gb.gui.base");
+	
+	GB.GetInterface("gb.geom", GEOM_INTERFACE_VERSION, &GEOM);
 	GB.GetInterface("gb.image", IMAGE_INTERFACE_VERSION, &IMAGE);
+	
   IMAGE.SetDefaultFormat(GB_IMAGE_BGRP);
+	
 	DRAW_init();
 
 	CLASS_Control = GB.FindClass("Control");
