@@ -74,19 +74,24 @@ static QMap<int, int> _x11_to_qt_keycode;
 
 CWIDGET *CWIDGET_active_control = 0;
 CWIDGET *CWIDGET_previous_control = 0;
-static bool _focus_change = false;
-static CWIDGET *_old_active_control = 0;
+CWIDGET *CWIDGET_hovered = 0;
+
 int CCONTROL_last_event_type = 0;
 
+static bool _focus_change = false;
+static CWIDGET *_old_active_control = 0;
 static CWIDGET *_hovered = 0;
 static CWIDGET *_official_hovered = 0;
 
-QSet<CWIDGET *> *_enter_leave_set = NULL;
+#ifdef QT5
+static CWIDGET *_last_entered = NULL;
+#else
+static QSet<CWIDGET *> *_enter_leave_set = NULL;
+#endif
 
 static QT_COLOR_FUNC _after_set_color = NULL;
 
-#define EXT(_ob) (((CWIDGET *)_ob)->ext)
-
+#define EXT(_ob) ((CWIDGET_EXT *)((CWIDGET *)_ob)->ext)
 #define ENSURE_EXT(_ob) (EXT(_ob) ? EXT(_ob) : alloc_ext((CWIDGET *)(_ob)))
 
 #define HANDLE_PROXY(_ob) \
@@ -96,11 +101,11 @@ static QT_COLOR_FUNC _after_set_color = NULL;
 static CWIDGET_EXT *alloc_ext(CWIDGET *_object)
 {
 	GB.Alloc(POINTER(&(THIS->ext)), sizeof(CWIDGET_EXT));
-	CLEAR(THIS->ext);
+	CLEAR(THIS_EXT);
 	THIS_EXT->bg = COLOR_DEFAULT;
 	THIS_EXT->fg = COLOR_DEFAULT;
 	THIS_EXT->tag.type = GB_T_NULL;
-	return THIS->ext;
+	return THIS_EXT;
 }
 	
 static void set_mouse(QWidget *w, int mouse, void *cursor)
@@ -343,9 +348,17 @@ bool CWIDGET_container_for(void *_object, void *container_for)
 
 static void CWIDGET_enter(void *_object)
 {
-	//qDebug("CWIDGET_enter: %p %s", THIS, THIS->name);
+	CWIDGET *parent = (CWIDGET *)CWIDGET_get_parent(THIS);
+	
+	if (parent && !parent->flag.inside)
+		CWIDGET_enter(parent);
+	
 	if (!THIS->flag.inside)
 	{
+		//qDebug("CWIDGET_enter: %p %s", THIS, THIS->name);
+#ifdef QT5
+		_last_entered = THIS;
+#endif
 		THIS->flag.inside = true;
 		GB.Raise(THIS, EVENT_Enter, 0);
 	}
@@ -353,9 +366,13 @@ static void CWIDGET_enter(void *_object)
 
 static void CWIDGET_leave(void *_object)
 {
-	//qDebug("CWIDGET_leave: %p %s", THIS, THIS->name);
+#ifdef QT5
+		if (_last_entered == THIS)
+			_last_entered = (CWIDGET *)CWIDGET_get_parent((void *)_last_entered);
+#endif
 	if (THIS->flag.inside)
 	{
+		//qDebug("CWIDGET_leave: %p %s", THIS, THIS->name);
 		THIS->flag.inside = false;
 		GB.Raise(THIS, EVENT_Leave, 0);
 	}
@@ -421,6 +438,13 @@ QString CWIDGET_Utf8ToQString(GB_STRING *str)
 	return QString::fromUtf8((const char *)(str->value.addr + str->value.start), str->value.len);
 }
 
+#ifdef QT5
+void CWIDGET_leave_popup(void *)
+{
+	while (_last_entered)
+		CWIDGET_leave(_last_entered);
+}
+#else
 void *CWIDGET_enter_popup()
 {
 	void *save = _enter_leave_set;
@@ -464,13 +488,17 @@ static void insert_enter_leave_event(CWIDGET *control, bool in)
 	_enter_leave_set->insert(control);
 	GB.Ref(control);
 }
+#endif
 
 static bool _post_check_hovered = false;
 static CWIDGET *_post_check_hovered_window = NULL;
 
 static void post_check_hovered(intptr_t)
 {
-	CWIDGET *_object = _post_check_hovered_window;
+	void *_object = _post_check_hovered_window;
+	
+	if (!THIS)
+		_object = CWINDOW_Active;
 	
 	if (THIS && WIDGET)
 	{
@@ -483,6 +511,14 @@ static void post_check_hovered(intptr_t)
 	}
 	
 	_post_check_hovered = false;
+	_post_check_hovered_window = NULL;
+	
+}
+
+void CWIDGET_check_hovered()
+{
+	_post_check_hovered_window = NULL;
+	post_check_hovered(0);
 }
 
 void CWIDGET_destroy(CWIDGET *_object)
@@ -499,6 +535,11 @@ void CWIDGET_destroy(CWIDGET *_object)
 		return;
 	}
 
+#ifdef QT5
+	if (_last_entered == THIS)
+		_last_entered = NULL;
+#endif
+	
 	//qDebug("CWIDGET_destroy: %s %p", GB.GetClassName(THIS), THIS);
 
 	CWIDGET_set_visible(THIS, false);
@@ -2268,12 +2309,11 @@ void CWidget::destroy()
 	if (!_post_check_hovered)
 	{
 		CWIDGET *top = (CWIDGET *)CWidget::getTopLevel(THIS);
-		if (top != THIS)
-		{
-			_post_check_hovered = true;
-			_post_check_hovered_window = top;
-			GB.Post((void (*)())post_check_hovered, NULL);
-		}
+		if (top == THIS)
+			top = NULL;
+		_post_check_hovered = true;
+		_post_check_hovered_window = top;
+		GB.Post((void (*)())post_check_hovered, (intptr_t)0);
 	}
 	
 	CLEAN_POINTER(_hovered);
@@ -2281,6 +2321,7 @@ void CWidget::destroy()
 	CLEAN_POINTER(_post_check_hovered_window);
 	CLEAN_POINTER(CWIDGET_active_control);
 	CLEAN_POINTER(CWIDGET_previous_control);
+	CLEAN_POINTER(CWIDGET_hovered);
 	CLEAN_POINTER(_old_active_control);
 
 	if (THIS_EXT)
@@ -2367,7 +2408,7 @@ static void handle_focus_change()
 		return;
 	
 	_focus_change = TRUE;
-	GB.Post((void (*)())post_focus_change, NULL);
+	GB.Post((void (*)())post_focus_change, (intptr_t)NULL);
 }
 
 void CWIDGET_finish_focus(void)
@@ -2490,16 +2531,22 @@ bool CWidget::eventFilter(QObject *widget, QEvent *event)
 
 	__ENTER:
 	{
+#ifndef QT5
 		QWidget *popup = qApp->activePopupWidget();
+#endif
 		
-		//qDebug("enter %s real = %d popup = %p inside = %d", control->name, real, popup, control->flag.inside);
+		//qDebug("enter %p %s real = %d inside = %d", widget, control->name, real, control->flag.inside);
 		
 		if (real)
 		{
+#ifdef QT5
+			CWIDGET_enter(control);
+#else
 			if (!popup || CWidget::getReal(popup))
 				CWIDGET_enter(control);
 			else if (_enter_leave_set)
 				insert_enter_leave_event(control, true);
+#endif
 		}
 		
 		goto __NEXT;
@@ -2507,16 +2554,22 @@ bool CWidget::eventFilter(QObject *widget, QEvent *event)
 
 	__LEAVE:
 	{
+#ifndef QT5
 		QWidget *popup = qApp->activePopupWidget();
+#endif
 		
-		//qDebug("leave %s real = %d popup = %p inside = %d", control->name, real, popup, control->flag.inside);
+		//qDebug("leave %p %s real = %d inside = %d", widget, control->name, real, control->flag.inside);
 		
 		if (real)
 		{
+#ifdef QT5
+			CWIDGET_leave(control);
+#else
 			if (!popup || CWidget::getReal(popup))
 				CWIDGET_leave(control);
 			else if (_enter_leave_set)
 				insert_enter_leave_event(control, false);
+#endif
 		}
 		
 		goto __NEXT;
