@@ -21,10 +21,11 @@
 
 #include "utils.h"
 #include "gbinterface.h"
-#include "../gb_common.h"
+#include "gb_common.h"
 #include "textnode.h"
 
 #include <stdlib.h>
+#include <stdio.h>
 #include <memory.h>
 
 #ifdef OS_MACOSX
@@ -138,7 +139,7 @@ const void* memchrs(const char *source, size_t lensource, const char *comp, size
     const char *pos = source - 1;
     do
     {
-        pos = (char*)(memchr((void*)(pos + 1), ((comp))[0], lensource - (pos - source)));
+        pos = (char*)(memchr((void*)(pos + 1), ((comp))[0], lensource - (pos - source) - 1));
         if(!pos) return 0;
         if(pos + lencomp > source + lensource) return 0;
         if(memcmp(pos, comp, lencomp) != 0) continue;
@@ -250,7 +251,7 @@ void XML_Format(GB_VALUE *value, char* &dst, size_t &lenDst)
         break;
 
     case GB_T_LONG:
-        lenDst = sprintf(buffer, "%ld", VALUE((GB_LONG *)value));
+        lenDst = sprintf(buffer, "%" PRId64, VALUE((GB_LONG *)value));
         break;
 
     case GB_T_STRING:
