@@ -2,7 +2,7 @@
 
   gb_str.h
 
-  (c) 2000-2012 Benoît Minisini <gambas@users.sourceforge.net>
+  (c) 2000-2013 Benoît Minisini <gambas@users.sourceforge.net>
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -31,6 +31,6 @@ char *STR_copy_len(const char *str, int len);
 char *STR_cat(const char *str, ...);
 char *STR_add(char *d, const char *s);
 
-#define STR_free(_str) IFREE(_str, "STR_free")
+#define STR_free(_str) IFREE(_str)
 
 #endif
