@@ -35,12 +35,67 @@
 #include "gbc_trans.h"
 #include "gb_code.h"
 
-//#define DEBUG
+//#define DEBUG 1
 
 //static bool _accept_statement = FALSE;
 
 static bool _must_drop_vargs = FALSE;
 
+static TYPE _type[MAX_EXPR_LEVEL];
+static int _type_level = 0;
+static TYPE _last_type;
+
+static void trans_expression(bool check_statement);
+
+#if DEBUG
+
+static void _push_type(TYPE type)
+{
+	if (_type_level >= MAX_EXPR_LEVEL) // should have been detected by TRANS_tree()
+		THROW("Expression too complex");
+	
+	fprintf(stderr, "level = %d / %d\n", _type_level, MAX_EXPR_LEVEL);
+	_type[_type_level++] = type;
+}
+
+static void _drop_type(int n)
+{
+	_type_level -= n;
+	if (_type_level < 0)
+		ERROR_panic("Incorrect type analyze");
+}
+
+#define push_type(_type) fprintf(stderr, "push_type: %d in %s.%d\n", (_type).t.id, __func__, __LINE__), _push_type(_type)
+#define push_type_id(_id) fprintf(stderr, "push_type_id: %d in %s.%d\n", (_id), __func__, __LINE__), _push_type(TYPE_make_simple(_id))
+#define pop_type() (fprintf(stderr, "pop_type: in %s.%d\n", __func__, __LINE__),(_type[--_type_level]))
+#define drop_type(_n) fprintf(stderr, "drop_type: %d in %s.%d\n", (_n), __func__, __LINE__),_drop_type(_n)
+#define get_type(_i, _nparam) (fprintf(stderr, "get_type(%d,%d): %d in %s.%d\n", (_i), (_nparam), (_type[_type_level + (_i) - (_nparam)].t.id), __func__, __LINE__),(_type[_type_level + (_i) - (_nparam)].t.id))
+#define dup_type() fprintf(stderr, "dup_type: in %s.%d\n", __func__, __LINE__),push_type(_type[_type_level - 1])
+
+#else
+
+static void push_type(TYPE type)
+{
+	if (_type_level >= MAX_EXPR_LEVEL) // should have been detected by TRANS_tree()
+		THROW("Expression too complex");
+	
+	_type[_type_level++] = type;
+}
+
+static void drop_type(int n)
+{
+	_type_level -= n;
+	if (_type_level < 0)
+		ERROR_panic("Incorrect type analyze");
+}
+
+#define push_type_id(_id) push_type(TYPE_make_simple(_id))
+#define pop_type() (_type[--_type_level])
+#define get_type(_i, _nparam) (_type[_type_level + (_i) - (_nparam)].t.id)
+#define dup_type() push_type(_type[_type_level - 1])
+
+#endif
+
 static short get_nparam(PATTERN *tree, int count, int *pindex, uint64_t *byref)
 {
 	PATTERN pattern;
@@ -80,7 +135,7 @@ static short get_nparam(PATTERN *tree, int count, int *pindex, uint64_t *byref)
 				index++;
 			}
 		}
-		
+
 		*pindex = index;
 	}
 
@@ -108,16 +163,21 @@ static void push_number(int index)
 	else
 	{
 		CLEAR(&decl);
-		decl.type = TYPE_make(number.type, 0, 0);
+		decl.type = TYPE_make_simple(number.type);
 		decl.index = NO_SYMBOL;
 		decl.value = index;
 		if (number.type == T_LONG)
 			decl.lvalue = number.lval;
 		CODE_push_const(CLASS_add_constant(JOB->class, &decl));
 	}
-	
+
 	if (number.complex)
+	{
 		CODE_push_complex();
+		push_type_id(T_OBJECT);
+	}
+	else
+		push_type_id(number.type);
 }
 
 
@@ -148,14 +208,16 @@ static void push_string(int index, bool trans)
 		//CLEAR(&decl);
 
 		if (trans)
-			decl.type = TYPE_make(T_CSTRING, 0, 0);
+			decl.type = TYPE_make_simple(T_CSTRING);
 		else
-			decl.type = TYPE_make(T_STRING, 0, 0);
+			decl.type = TYPE_make_simple(T_STRING);
 		decl.index = NO_SYMBOL;
 		decl.value = index;
 
 		CODE_push_const(CLASS_add_constant(JOB->class, &decl));
 	}
+	
+	push_type_id(T_STRING);
 }
 
 bool TRANS_string(PATTERN pattern)
@@ -167,17 +229,18 @@ bool TRANS_string(PATTERN pattern)
 }
 
 
-void TRANS_class(int index)
+static void trans_class(int index)
 {
 	const char *name;
 
-	if (CLASS_exist_class(JOB->class, index))
-		CODE_push_class(CLASS_add_class(JOB->class, index));
-	else
+	if (!CLASS_exist_class(JOB->class, index))
 	{
 		name = TABLE_get_symbol_name(JOB->class->table, index);
 		THROW("Unknown identifier: &1", name);
 	}
+	
+	CODE_push_class(CLASS_add_class(JOB->class, index));
+	push_type_id(T_OBJECT);
 }
 
 static void trans_identifier(int index, bool point, PATTERN next)
@@ -188,9 +251,14 @@ static void trans_identifier(int index, bool point, PATTERN next)
 	int type;
 	CONSTANT *constant;
 
+#if DEBUG
+	fprintf(stderr, "trans_identifier: %.*s\n", sym->symbol.len, sym->symbol.name);
+#endif
+	
 	if (!TYPE_is_null(sym->local.type) && !point)
 	{
 		CODE_push_local(sym->local.value);
+		push_type(sym->local.type);
 		sym->local_used = TRUE;
 	}
 	else if (!TYPE_is_null(sym->global.type) && !point)
@@ -212,6 +280,8 @@ static void trans_identifier(int index, bool point, PATTERN next)
 				CODE_push_number(constant->value);
 			else
 				CODE_push_const(sym->global.value);
+			
+			push_type_id(type);
 		}
 		else if (type == TK_EXTERN)
 		{
@@ -219,6 +289,7 @@ static void trans_identifier(int index, bool point, PATTERN next)
 				goto __CLASS;
 
 			CODE_push_extern(sym->global.value);
+			push_type(JOB->class->ext_func[sym->global.value].type);
 		}
 		/* That breaks some code if the property has the same name as a class!
 		else if (type == TK_PROPERTY)
@@ -245,15 +316,22 @@ static void trans_identifier(int index, bool point, PATTERN next)
 				goto __CLASS;
 			else
 				CODE_push_global(sym->global.value, is_static, is_func);
+				
+			if (is_func)
+				push_type(JOB->class->function[sym->global.value].type);
+			else if (is_static)
+				push_type(JOB->class->stat[sym->global.value].type);
+			else
+				push_type(JOB->class->dyn[sym->global.value].type);
 		}
 	}
 	else
 	{
-		/*index = CLASS_add_symbol(JOB->class, index);*/
-
 		if (point)
+		{
 			CODE_push_unknown(CLASS_add_unknown(JOB->class, index));
-		/* Class must be declared now ! */
+			push_type_id(T_VOID);
+		}
 		else
 			goto __CLASS;
 	}
@@ -262,13 +340,14 @@ static void trans_identifier(int index, bool point, PATTERN next)
 
 __CLASS:
 
-	TRANS_class(index);
+	trans_class(index);
 }
 
 
 static void trans_subr(int subr, short nparam)
 {
 	SUBR_INFO *info = &COMP_subr_info[subr];
+	int type;
 
 	if (nparam < info->min_param)
 		THROW("Not enough arguments to &1()", info->name);
@@ -288,20 +367,53 @@ static void trans_subr(int subr, short nparam)
 	}
 
 	CODE_subr(info->opcode, nparam, info->optype, info->max_param == info->min_param);
+	
+	type = info->type;
+	switch(type)
+	{
+		case RST_SAME:
+			type = get_type(0, nparam);
+			break;
+			
+		case RST_BCLR:
+			type = get_type(0, nparam);
+			break;
+	}
+	
+	if (nparam)
+		drop_type(nparam);
+	
+	push_type_id(type);
 }
 
 
 static void trans_operation(short op, short nparam, PATTERN previous)
 {
 	COMP_INFO *info = &COMP_res_info[op];
+	int type = info->type;
 
 	switch (info->value)
 	{
 		case OP_PT:
 			if (nparam == 0)
+			{
 				TRANS_use_with();
+				type = T_OBJECT;
+			}
 			else if (!PATTERN_is_identifier(previous))
 				THROW(E_SYNTAX);
+			else
+			{
+				switch(get_type(0, nparam))
+				{
+					case T_OBJECT:
+					case T_VARIANT:
+						break;
+						
+					default:
+						THROW("Not an object");
+				}
+			}
 
 			break;
 
@@ -316,22 +428,64 @@ static void trans_operation(short op, short nparam, PATTERN previous)
 
 		case OP_RSQR:
 			TRANS_subr(TS_SUBR_ARRAY, nparam);
+			if (nparam > MAX_PARAM_OP)
+				nparam--;
 			break;
 
 		case OP_COLON:
 			TRANS_subr(TS_SUBR_COLLECTION, nparam);
+			if (nparam > MAX_PARAM_OP)
+				nparam -= 2;
 			break;
 
 		case OP_MINUS:
 			if (nparam == 1)
+			{
 				CODE_op(C_NEG, 0, nparam, TRUE);
+				type = RST_SAME;
+			}
 			else
 				CODE_op(info->code, info->subcode, nparam, TRUE);
 			break;
-			
+
 		default:
 			CODE_op(info->code, info->subcode, nparam, (info->flag != RSF_OPN));
 	}
+	
+	switch(type)
+	{
+		case RST_SAME:
+			type = get_type(0, nparam);
+			break;
+			
+		case RST_ADD:
+			type = Max(get_type(0, nparam), get_type(1, nparam));
+			if (type == T_DATE)
+				type = T_FLOAT;
+			break;
+			
+		case RST_AND:
+			type = Max(get_type(0, nparam), get_type(1, nparam));
+			if (type > T_LONG && type != T_VARIANT && type != T_STRING)
+				THROW("Integer expected");
+			break;
+
+		case RST_NOT:
+			type = get_type(0, nparam);
+			if (type == T_STRING || type == T_OBJECT)
+				type = T_BOOLEAN;
+			break;
+			
+		case RST_MIN:
+			type = Max(get_type(0, nparam), get_type(1, nparam));
+			if (type > T_DATE && type != T_VARIANT)
+				THROW("Number or Date expected");
+	}
+	
+	if (nparam)
+		drop_type(nparam);
+	
+	push_type_id(type);
 }
 
 
@@ -340,136 +494,175 @@ static void trans_call(short nparam, uint64_t byref)
 	if (!byref)
 	{
 		CODE_call(nparam);
-		return;
 	}
-		
-	CODE_call_byref(nparam, byref);
-
-	if (_must_drop_vargs)
+	else
 	{
-		CODE_drop_vargs();
-		_must_drop_vargs = FALSE;
+		CODE_call_byref(nparam, byref);
+
+		if (_must_drop_vargs)
+		{
+			CODE_drop_vargs();
+			_must_drop_vargs = FALSE;
+		}
 	}
+	
+	drop_type(nparam + 1);
+	
+	push_type_id(T_VARIANT);
 }
 
 static void trans_expr_from_tree(TRANS_TREE *tree, int count)
 {
+	static void *jump[] = {
+		&&__CONTINUE, &&__CONTINUE, &&__RESERVED, &&__IDENTIFIER, &&__NUMBER, &&__STRING, &&__TSTRING, &&__CONTINUE, &&__SUBR, &&__CLASS, &&__CONTINUE, &&__CONTINUE
+	};
+	
 	int i, op;
 	short nparam;
 	PATTERN pattern, next_pattern, prev_pattern;
-	uint64_t byref;
+	uint64_t byref = 0;
 
-	count--;
 	pattern = NULL_PATTERN;
 
-	for (i = 0; i <= count; i++)
+	#if DEBUG
+	fprintf(stderr, "-------------------------------------------- %d\n", _type_level);
+	#endif
+
+	i = 0;
+	
+	for(i = 0; i < count; i++)
 	{
 		prev_pattern = pattern;
 		pattern = tree[i];
-		if (i < count)
-			next_pattern = tree[i + 1];
-		else
-			next_pattern = NULL_PATTERN;
+		next_pattern = tree[i + 1];
 
-		if (PATTERN_is_number(pattern))
-			push_number(PATTERN_index(pattern));
+		goto *jump[PATTERN_type(pattern)];
+		
+	__NUMBER:
+		
+		push_number(PATTERN_index(pattern));
+		continue;
+		
+	__STRING:
 
-		else if (PATTERN_is_string(pattern))
-			push_string(PATTERN_index(pattern), FALSE);
+		push_string(PATTERN_index(pattern), FALSE);
+		continue;
 
-		else if (PATTERN_is_tstring(pattern))
-			push_string(PATTERN_index(pattern), TRUE);
+	__TSTRING:
 
-		else if (PATTERN_is_identifier(pattern))
-			trans_identifier(PATTERN_index(pattern), PATTERN_is_point(pattern), next_pattern);
+		push_string(PATTERN_index(pattern), TRUE);
+		continue;
 
-		else if (PATTERN_is_class(pattern))
-			TRANS_class(PATTERN_index(pattern));
+	__IDENTIFIER:
+	
+		trans_identifier(PATTERN_index(pattern), PATTERN_is_point(pattern), next_pattern);
+		continue;
 
-		else if (PATTERN_is_subr(pattern))
+	__CLASS:
+	
+		trans_class(PATTERN_index(pattern));
+		continue;
+
+	__SUBR:
+	
+		nparam = get_nparam(tree, count, &i, NULL);
+		trans_subr(PATTERN_index(pattern), nparam);
+		continue;
+		
+	__RESERVED:
+
+		if (PATTERN_is(pattern, RS_TRUE))
 		{
-			nparam = get_nparam(tree, count, &i, NULL);
-			trans_subr(PATTERN_index(pattern), nparam);
+			CODE_push_boolean(TRUE);
+			push_type_id(T_BOOLEAN);
 		}
-
-		else if (PATTERN_is_reserved(pattern))
+		else if (PATTERN_is(pattern, RS_FALSE))
 		{
-			if (PATTERN_is(pattern, RS_TRUE))
-			{
-				CODE_push_boolean(TRUE);
-			}
-			else if (PATTERN_is(pattern, RS_FALSE))
-			{
-				CODE_push_boolean(FALSE);
-			}
-			else if (PATTERN_is(pattern, RS_NULL))
-			{
-				CODE_push_null();
-			}
-			else if (PATTERN_is(pattern, RS_ME))
-			{
-				/*if (FUNCTION_is_static(JOB->func))
-					THROW("ME cannot be used in a static function");*/
+			CODE_push_boolean(FALSE);
+			push_type_id(T_BOOLEAN);
+		}
+		else if (PATTERN_is(pattern, RS_NULL))
+		{
+			CODE_push_null();
+			push_type_id(T_OBJECT);
+		}
+		else if (PATTERN_is(pattern, RS_ME))
+		{
+			/*if (FUNCTION_is_static(JOB->func))
+				THROW("ME cannot be used in a static function");*/
 
-				CODE_push_me(FALSE);
-			}
-			else if (PATTERN_is(pattern, RS_SUPER))
-			{
-				/*if (FUNCTION_is_static(JOB->func))
-					THROW("ME cannot be used in a static function");*/
+			CODE_push_me(FALSE);
+			push_type_id(T_OBJECT);
+		}
+		else if (PATTERN_is(pattern, RS_SUPER))
+		{
+			/*if (FUNCTION_is_static(JOB->func))
+				THROW("ME cannot be used in a static function");*/
 
-				CODE_push_super(FALSE);
-			}
-			else if (PATTERN_is(pattern, RS_LAST))
+			CODE_push_super(FALSE);
+			push_type_id(T_OBJECT);
+		}
+		else if (PATTERN_is(pattern, RS_LAST))
+		{
+			CODE_push_last();
+			push_type_id(T_OBJECT);
+		}
+		else if (PATTERN_is(pattern, RS_AT))
+		{
+			if (!CODE_popify_last())
+				THROW("This expression cannot be passed by reference");
+		}
+		else if (PATTERN_is(pattern, RS_COMMA))
+		{
+			CODE_drop();
+			drop_type(1);
+		}
+		else if (PATTERN_is(pattern, RS_ERROR))
+		{
+			TRANS_subr(TS_SUBR_ERROR, 0);
+			push_type_id(T_BOOLEAN);
+		}
+		else if (PATTERN_is(pattern, RS_OPTIONAL))
+		{
+			CODE_push_void();
+			push_type_id(T_VOID);
+		}
+		else if (PATTERN_is(pattern, RS_PINF))
+		{
+			CODE_push_inf(FALSE);
+			push_type_id(T_FLOAT);
+		}
+		else if (PATTERN_is(pattern, RS_MINF))
+		{
+			CODE_push_inf(TRUE);
+			push_type_id(T_FLOAT);
+		}
+		else if (PATTERN_is(pattern, RS_3PTS))
+		{
+			_must_drop_vargs = TRUE;
+			CODE_push_vargs();
+			// push_type_id(T_FLOAT); we push no type
+		}
+		else
+		{
+			op = PATTERN_index(pattern);
+			if (op == RS_LBRA)
 			{
-				CODE_push_last();
-			}
-			else if (PATTERN_is(pattern, RS_AT))
-			{
-				if (!CODE_popify_last())
-					THROW("This expression cannot be passed by reference");
-			}
-			else if (PATTERN_is(pattern, RS_COMMA))
-			{
-				CODE_drop();
-			}
-			else if (PATTERN_is(pattern, RS_ERROR))
-			{
-				TRANS_subr(TS_SUBR_ERROR, 0);
-			}
-			else if (PATTERN_is(pattern, RS_OPTIONAL))
-			{
-				CODE_push_void();
-			}
-			else if (PATTERN_is(pattern, RS_PINF))
-			{
-				CODE_push_inf(FALSE);
-			}
-			else if (PATTERN_is(pattern, RS_MINF))
-			{
-				CODE_push_inf(TRUE);
-			}
-			else if (PATTERN_is(pattern, RS_3PTS))
-			{
-				_must_drop_vargs = TRUE;
-				CODE_push_vargs();
+				nparam = get_nparam(tree, count, &i, &byref);
+				trans_call(nparam, byref);
 			}
 			else
 			{
-				op = PATTERN_index(pattern);
-				if (op == RS_LBRA)
-				{
-					nparam = get_nparam(tree, count, &i, &byref);
-					trans_call(nparam, byref);
-				}
-				else
-				{
-					nparam = get_nparam(tree, count, &i, NULL);
-					trans_operation((short)op, nparam, prev_pattern);
-				}
+				nparam = get_nparam(tree, count, &i, NULL);
+				trans_operation((short)op, nparam, prev_pattern);
 			}
 		}
+		
+	__CONTINUE:
+		;
 	}
+	
+	_last_type = pop_type();
 }
 
 
@@ -489,7 +682,7 @@ void TRANS_new(void)
 		index = CLASS_add_class(JOB->class, PATTERN_index(*JOB->current));
 		if (PATTERN_is(JOB->current[1], RS_LSQR))
 			index = CLASS_get_array_class(JOB->class, T_OBJECT, index);
-		
+
 		CODE_push_class(index);
 		nparam = 1;
 	}
@@ -592,7 +785,7 @@ void TRANS_new(void)
 }
 
 
-void TRANS_expression(bool check_statement)
+static void trans_expression(bool check_statement)
 {
 	TRANS_TREE *tree;
 	int tree_length;
@@ -610,7 +803,7 @@ void TRANS_expression(bool check_statement)
 			return;
 		}
 	}
-	
+
 	TRANS_tree(check_statement, &tree, &tree_length);
 
 	trans_expr_from_tree(tree, tree_length);
@@ -635,14 +828,14 @@ void TRANS_ignore_expression()
 
 TYPE TRANS_variable_get_type()
 {
-	TYPE type = TYPE_make(T_VOID, 0, 0);
+	TYPE type = TYPE_make_simple(T_VOID);
 	TRANS_TREE *tree;
 	int count;
 	int index;
 	CLASS_SYMBOL *sym;
-	
+
 	TRANS_tree(FALSE, &tree, &count);
-	
+
 	if (count == 1 && PATTERN_is_identifier(*tree))
 	{
 		index = PATTERN_index(*tree);
@@ -659,9 +852,9 @@ TYPE TRANS_variable_get_type()
 				return sym->global.type;
 		}
 	}
-	
+
 	FREE(&tree);
-	
+
 	return type;
 }
 
@@ -699,7 +892,7 @@ bool TRANS_affectation(bool dup)
 	bool stat = FALSE;
 	int op = RS_NONE;
 	int id = RS_NONE;
-
+	
 	for(;;)
 	{
 		if (PATTERN_is_newline(*look)) // || PATTERN_is_end(*look))
@@ -771,21 +964,25 @@ bool TRANS_affectation(bool dup)
 
 	if (!stat)
 	{
+		TYPE type;
+		
 		if (op != RS_NONE)
 		{
 			JOB->current = left;
-			TRANS_expression(FALSE);
+			trans_expression(FALSE);
+			type = _last_type;
 		}
 
 		JOB->current = expr;
-		TRANS_expression(FALSE);
+		trans_expression(FALSE);
 		after = JOB->current;
 
 		if (op != RS_NONE)
 		{
-			/*if (op == RS_AMP)
-				CODE_string_add();*/
+			push_type(_last_type);
+			push_type(type);
 			trans_operation(op, 2, NULL_PATTERN);
+			pop_type();
 		}
 	}
 
@@ -821,3 +1018,9 @@ bool TRANS_affectation(bool dup)
 }
 
 
+void TRANS_expression(bool check_statement)
+{
+	_type_level = 0;
+	trans_expression(check_statement);
+}
+
