@@ -156,6 +156,7 @@ void ERROR_propagate(void) NORETURN;
 
 void THROW(int code, ...) NORETURN;
 void THROW_SYSTEM(int err, const char *path);
+void THROW_NULL(void) NORETURN;
 void THROW_ILLEGAL(void) NORETURN;
 void THROW_STACK(void) NORETURN;
 void THROW_CLASS(void *class, char *arg1, char *arg2) NORETURN;
@@ -171,7 +172,6 @@ void ERROR_hook(void);
 void ERROR_save(ERROR_INFO *save, ERROR_INFO *last);
 void ERROR_restore(ERROR_INFO *save, ERROR_INFO *last);
 
-void ERROR_clear(void);
 void ERROR_reset(ERROR_INFO *info);
 void ERROR_lock(void);
 void ERROR_unlock(void);
