@@ -130,6 +130,7 @@ const void *const GAMBAS_Api[] =
 	(void *)CLASS_find_global,
 	(void *)GB_ExistClassLocal,
 	(void *)CLASS_find,
+	(void *)GB_GetArrayType,
 	(void *)GB_Is,
 	(void *)GB_Ref,
 	(void *)GB_Unref,
@@ -253,7 +254,7 @@ const void *const GAMBAS_Api[] =
 	(void *)GB_SystemPath,
 	(void *)FILE_init,
 	(void *)GB_SystemDebug,
-	(void *)PROJECT_get_home,
+	(void *)FILE_get_home,
 
 	(void *)GB_ArrayNew,
 	(void *)GB_ArrayCount,
@@ -506,9 +507,9 @@ static int _raise_event_level = 0;
 	CATCH \
 	{ \
 		ret = TRUE; \
-		EXEC_set_native_error(TRUE); \
 	} \
 	END_TRY \
+	if (ret) EXEC_set_native_error(TRUE); \
 	return ret;
 
 #define CATCH_ERROR_INT \
@@ -519,9 +520,9 @@ static int _raise_event_level = 0;
 	CATCH \
 	{ \
 		ret = -1; \
-		EXEC_set_native_error(TRUE); \
 	} \
 	END_TRY \
+	if (ret < 0) EXEC_set_native_error(TRUE); \
 	return ret;
 
 bool GB_GetInterface(const char *name, int version, void *iface)
@@ -872,7 +873,10 @@ static bool raise_event(OBJECT *observer, void *object, int func_id, int nparam)
 	func_id--;
 
 	if (OBJECT_is_class(observer))
+	{
 		class = (CLASS *)observer; //OBJECT_class(object);
+		observer = NULL;
+	}
 	else
 		class = OBJECT_class(observer);
 
@@ -1812,7 +1816,7 @@ void *GB_GetClass(void *object)
 char *GB_GetClassName(void *object)
 {
 	CLASS *class = GB_GetClass(object);
-	return CLASS_get_name(class);
+	return class == CLASS_Class ? CLASS_get_name((CLASS *)object) : CLASS_get_name(class);
 }
 
 
@@ -1847,11 +1851,13 @@ bool GB_LoadFile(const char *path, int lenp, char **addr, int *len)
 		if (*addr)
 			STREAM_unmap(*addr, *len);
 
-		EXEC_set_native_error(TRUE);
 		ret = TRUE;
 	}
 	END_TRY
 
+	if (ret)
+		EXEC_set_native_error(TRUE);
+	
 	return ret;
 }
 
@@ -2457,6 +2463,11 @@ bool GB_ExistClassLocal(const char *name)
 	return CLASS_look(name, strlen(name)) != NULL;
 }
 
+TYPE GB_GetArrayType(void *klass)
+{
+  return ((CLASS *)klass)->array_type;
+}
+
 void GB_Wait(int delay)
 {
 	struct timespec rem;
