@@ -49,6 +49,17 @@ static void error(int code, CLASS *class, const char *name)
 	GB_Error((char *)(intptr_t)code, CLASS_get_name(class), name);
 }
 
+static bool check_null(void *object)
+{
+	if (!object)
+	{
+		GB_Error((char *)E_NULL);
+		return TRUE;
+	}
+	
+	return FALSE;
+}
+
 //---- Components ---------------------------------------------------------
 
 BEGIN_METHOD(Components_get, GB_STRING name)
@@ -167,14 +178,14 @@ BEGIN_METHOD_VOID(Classes_next)
 	CLASS *class;
 
 	class = *pcurrent;
-	
+
 	for(;;)
 	{
 		if (!class)
 			class = CLASS_get_list();
 		else
 			class = class->next;
-		
+
 		if (!class)
 		{
 			GB_StopEnum();
@@ -186,11 +197,11 @@ BEGIN_METHOD_VOID(Classes_next)
 			GB_ReturnObject(class);
 			break;
 		}
-		
+
 	}
 
 	*pcurrent = class;
-	
+
 END_METHOD
 
 
@@ -251,9 +262,9 @@ BEGIN_PROPERTY(Class_Symbols)
 	GB_ARRAY array;
 	CLASS_DESC_SYMBOL *cds;
 	int index = 0;
-	
+
 	GB_ArrayNew(&array, T_STRING, 0);
-	
+
 	for(;;)
 	{
 		cds = CLASS_get_next_sorted_symbol(class, &index);
@@ -261,7 +272,7 @@ BEGIN_PROPERTY(Class_Symbols)
 			break;
 		*((char **)GB_ArrayAdd(array)) = STRING_new(cds->name, cds->len);
 	}
-	
+
 	GB_ReturnObject(array);
 
 END_PROPERTY
@@ -308,7 +319,7 @@ END_PROPERTY
 BEGIN_METHOD_VOID(Class_AutoCreate)
 
 	CLASS *class = OBJECT(CLASS);
-	
+
 	if (!class->auto_create)
 		GB_ReturnNull();
 	else
@@ -469,7 +480,7 @@ BEGIN_PROPERTY(Symbol_Value)
 		GB_ReturnConstZeroString(desc->constant.value._string);
 	else
 		GB_ReturnPtr(desc->constant.type, (void *)&desc->constant.value);
-	
+
 	GB_ReturnConvVariant();
 
 END_PROPERTY
@@ -527,7 +538,7 @@ BEGIN_METHOD(Object_Attach, GB_OBJECT object; GB_OBJECT parent; GB_STRING name)
 		return;
 
 	OBJECT_attach(object, parent, name);
-	
+
 	/*if (OBJECT_is(object, CLASS_Observer))
 		COBSERVER_attach((COBSERVER *)object, parent, GB_ToZeroString(ARG(name)));*/
 
@@ -543,7 +554,7 @@ BEGIN_METHOD(Object_Detach, GB_OBJECT object)
 
 	if (OBJECT_is(object, CLASS_Observer))
 		COBSERVER_detach((COBSERVER *)object);
-	
+
 	OBJECT_detach(object);
 
 END_METHOD
@@ -565,7 +576,7 @@ BEGIN_METHOD(Object_Class, GB_OBJECT object)
 
 	void *object = VARG(object);
 
-	if (GB_CheckObject(object))
+	if (check_null(object))
 		return;
 
 	GB_ReturnObject(OBJECT_class(object));
@@ -577,7 +588,7 @@ BEGIN_METHOD(Object_Type, GB_OBJECT object)
 
 	void *object = VARG(object);
 
-	if (GB_CheckObject(object))
+	if (check_null(object))
 		return;
 
 	GB_ReturnConstZeroString(OBJECT_class(object)->name);
@@ -590,7 +601,7 @@ BEGIN_METHOD(Object_Is, GB_OBJECT object; GB_STRING class)
 	void *object = VARG(object);
 	CLASS *class = CLASS_look(STRING(class), LENGTH(class));
 
-	if (GB_CheckObject(object))
+	if (check_null(object))
 		return;
 
 	if (!class)
@@ -615,7 +626,7 @@ BEGIN_METHOD(Object_Call, GB_OBJECT object; GB_STRING method; GB_OBJECT params)
 
 	if (GB_CheckObject(object))
 		return;
-	
+
 	if (GB_GetFunction(&func, object, name, NULL, NULL))
 	{
 		error(E_NSYMBOL, OBJECT_class(object), name);
@@ -637,7 +648,7 @@ BEGIN_METHOD(Object_Call, GB_OBJECT object; GB_STRING method; GB_OBJECT params)
 	}
 
 	GB_Call(&func, np, FALSE);
-	
+
 	if (TEMP.type != T_VOID)
 		GB_ReturnConvVariant();
 
@@ -656,13 +667,13 @@ BEGIN_METHOD(Object_Raise, GB_OBJECT object; GB_STRING event; GB_OBJECT params)
 
 	if (GB_CheckObject(object))
 		return;
-	
+
 	class = OBJECT_class(object);
-	
+
 	event[0] = ':';
 	memcpy(&event[1], STRING(event), len);
 	event[len + 1] = 0;
-	
+
 	index = CLASS_find_symbol(class, event);
 	if (index == NO_SYMBOL)
 		goto __UNKNOWN_EVENT;
@@ -670,7 +681,7 @@ BEGIN_METHOD(Object_Raise, GB_OBJECT object; GB_STRING event; GB_OBJECT params)
 	desc = class->table[index].desc;
 	if (CLASS_DESC_get_type(desc) != CD_EVENT)
 		goto __UNKNOWN_EVENT;
-	
+
 	if (params)
 		np = GB_ArrayCount(params);
 	else
@@ -689,11 +700,11 @@ BEGIN_METHOD(Object_Raise, GB_OBJECT object; GB_STRING event; GB_OBJECT params)
 
 	GB_ReturnBoolean(GB_Raise(object, desc->event.index, -np));
 	return;
-	
+
 __UNKNOWN_EVENT:
 
 	GB_Error("Unknown event");
-	
+
 END_METHOD
 
 
@@ -757,7 +768,7 @@ BEGIN_METHOD(Object_SizeOf, GB_OBJECT object)
 
 	void *object = VARG(object);
 
-	if (GB_CheckObject(object))
+	if (check_null(object))
 		return;
 
 	GB_ReturnInteger(CLASS_sizeof(OBJECT_class(object)));
@@ -827,7 +838,7 @@ BEGIN_METHOD(Object_CanRaise, GB_OBJECT object; GB_STRING name)
 		GB_ReturnBoolean(FALSE);
 	else
 		GB_ReturnBoolean(GB_CanRaise(object, index));
-	
+
 END_METHOD
 
 #endif
