@@ -46,7 +46,7 @@ static void clear(CCOLLECTION *col)
 
 	CLEAR(&iter);
 	col->locked = TRUE;
-	
+
 	for(;;)
 	{
 		value = HASH_TABLE_next(col->hash_table, &iter, FALSE);
@@ -82,7 +82,7 @@ static void remove_key(CCOLLECTION *col, const char *key, int len)
 		return;
 
 	last = col->hash_table->last;
-	
+
 	if (last)
 	{
 		save_enum = GB_BeginEnum(col);
@@ -96,7 +96,7 @@ static void remove_key(CCOLLECTION *col, const char *key, int len)
 	}
 
 	VARIANT_free((VARIANT *)value);
-	
+
 	if (!col->locked)
 		HASH_TABLE_remove(col->hash_table, key, len);
 	else // Prevent the freed variant to be freed twice if the collection is locked (i.e. being destroyed)
@@ -110,6 +110,7 @@ BEGIN_METHOD(Collection_new, GB_INTEGER mode)
 
 	HASH_TABLE_create(&THIS->hash_table, TYPE_sizeof(T_VARIANT), mode);
 	THIS->last = NULL;
+	THIS->default_value.type = GB_T_NULL;
 
 END_METHOD
 
@@ -117,6 +118,7 @@ END_METHOD
 BEGIN_METHOD_VOID(Collection_free)
 
 	clear(THIS);
+	GB_StoreVariant(NULL, POINTER(&THIS->default_value));
 
 END_METHOD
 
@@ -192,7 +194,7 @@ BEGIN_METHOD_VOID(Collection_next)
 	HASH_TABLE *hash_table = OBJECT(CCOLLECTION)->hash_table;
 	HASH_ENUM *iter = (HASH_ENUM *)GB_GetEnum();
 
-	value = HASH_TABLE_next(hash_table, iter, !DEBUG_inside_eval);
+	value = HASH_TABLE_next(hash_table, iter, TRUE);
 
 	if (value == NULL)
 		GB_StopEnum();
@@ -204,7 +206,14 @@ END_METHOD
 
 BEGIN_METHOD(Collection_get, GB_STRING key)
 
-	GB_ReturnVariant(get_key(THIS, STRING(key), LENGTH(key), !DEBUG_inside_eval));
+	void *value = get_key(THIS, STRING(key), LENGTH(key), TRUE);
+	if (!value)
+	{
+		if (THIS->has_default)
+			value = (void *)&THIS->default_value;
+	}
+
+	GB_ReturnVariant(value);
 
 END_METHOD
 
@@ -233,23 +242,37 @@ BEGIN_METHOD_VOID(Collection_Copy)
 	GB_VARIANT value;
 	char *key;
 	int len;
-	
+
 	GB_CollectionNew(&col, THIS->mode);
-	
+
 	CLEAR(&iter);
-	
+
 	for(;;)
 	{
 		if (GB_CollectionEnum(THIS, &iter, &value, &key, &len))
 			break;
-		
+
 		GB_CollectionSet(col, key, len, &value);
 	}
-	
+
 	GB_ReturnObject(col);
 
 END_METHOD
 
+
+BEGIN_PROPERTY(Collection_Default)
+
+	if (READ_PROPERTY)
+		GB_ReturnVariant(&THIS->default_value);
+	else
+	{
+		GB_StoreVariant(PROP(GB_VARIANT), POINTER(&THIS->default_value));
+		THIS->has_default = THIS->default_value.type != GB_T_NULL;
+	}
+
+END_PROPERTY
+
+
 #endif
 
 
@@ -263,6 +286,7 @@ GB_DESC NATIVE_Collection[] =
 	GB_PROPERTY_READ("Count", "i", Collection_Count),
 	GB_PROPERTY_READ("Length", "i", Collection_Count),
 	GB_PROPERTY("Key", "s", Collection_Key),
+	GB_PROPERTY("Default", "v", Collection_Default),
 
 	GB_METHOD("Add", NULL, Collection_Add, "(Value)v(Key)s"),
 	GB_METHOD("Exist", "b", Collection_Exist, "(Key)s"),
@@ -305,28 +329,40 @@ bool GB_CollectionSet(GB_COLLECTION col, const char *key, int len, GB_VARIANT *v
 		data = (VARIANT *)add_key((CCOLLECTION *)col, key, len);
 		if (!data)
 			return TRUE;
-		GB_StoreVariant(value, data);
+		VALUE_write_variant((VALUE *)value, data);
 	}
 	return FALSE;
 }
 
 bool GB_CollectionGet(GB_COLLECTION col, const char *key, int len, GB_VARIANT *value)
 {
+	CCOLLECTION *_object = (CCOLLECTION *)col;
 	VARIANT *var;
+	bool ret;
 
-	var = (VARIANT *)get_key((CCOLLECTION *)col, key, len, !DEBUG_inside_eval);
 	value->type = T_VARIANT;
-	if (var)
+
+	var = (VARIANT *)get_key(THIS, key, len, TRUE);
+
+	if (!var)
 	{
-		value->value.type = var->type;
-		value->value.value.data = var->value.data;
-		return FALSE;
+		if (!THIS->has_default)
+		{
+			value->value.type = GB_T_NULL;
+			return TRUE;
+		}
+
+		var = (VARIANT *)&THIS->default_value;
+		ret = TRUE;
 	}
 	else
 	{
-		value->value.type = GB_T_NULL;
-		return TRUE;
+		ret = FALSE;
 	}
+
+	value->value.type = var->type;
+	value->value.value.data = var->value.data;
+	return ret;
 }
 
 bool GB_CollectionEnum(GB_COLLECTION col, GB_COLLECTION_ITER *iter, GB_VARIANT *value, char **key, int *len)
@@ -340,7 +376,7 @@ bool GB_CollectionEnum(GB_COLLECTION col, GB_COLLECTION_ITER *iter, GB_VARIANT *
 		return FALSE;
 	}
 
-	val = HASH_TABLE_next(hash_table, (HASH_ENUM *)iter, !DEBUG_inside_eval);
+	val = HASH_TABLE_next(hash_table, (HASH_ENUM *)iter, TRUE);
 	if (!val)
 		return TRUE;
 
