@@ -31,6 +31,8 @@
 #include <pwd.h>
 #include <grp.h>
 #include <sys/stat.h>
+#include <pty.h>
+#include <termios.h>
 
 #include "gb_common.h"
 #include "gb_list.h"
@@ -45,19 +47,29 @@
 #include "gbx_string.h"
 #include "gbx_date.h"
 #include "gbx_watch.h"
+#include "gbx_signal.h"
 
 #include "gbx_c_file.h"
 
+#define STREAM_FD STREAM_handle(CSTREAM_stream(THIS_STREAM))
 
 CFILE *CFILE_in, *CFILE_out, *CFILE_err;
 
 DECLARE_EVENT(EVENT_Read);
 DECLARE_EVENT(EVENT_Write);
+DECLARE_EVENT(EVENT_Resize);
 
-static GB_FUNCTION read_func;
+static GB_FUNCTION _read_func;
 
 static char _buffer[16];
 
+static bool _term_init = FALSE;
+static ushort _term_width = 0;
+static ushort _term_height = 0;
+static SIGNAL_CALLBACK *_SIGWINCH_callback = NULL;
+static GB_FUNCTION _term_resize_func;
+
+
 static void callback_read(int fd, int type, CFILE *file)
 {
 	if (!STREAM_read_ahead(CSTREAM_stream(file)))
@@ -66,12 +78,18 @@ static void callback_read(int fd, int type, CFILE *file)
 		WATCH_little_sleep();
 }
 
-
 static void callback_write(int fd, int type, CFILE *file)
 {
 	GB_Raise(file, EVENT_Write, 0);
 }
 
+static void cb_term_resize(int signum, intptr_t data)
+{
+	_term_width = _term_height = 0;
+	if (CFILE_in)
+		GB_Raise(CFILE_in, EVENT_Resize, 0);
+}
+
 
 CFILE *CFILE_create(STREAM *stream, int mode)
 {
@@ -132,20 +150,27 @@ void CFILE_exit(void)
 	OBJECT_UNREF(CFILE_in);
 	OBJECT_UNREF(CFILE_out);
 	OBJECT_UNREF(CFILE_err);
+	
+	if (_term_init)
+		SIGNAL_unregister(SIGWINCH, _SIGWINCH_callback);
 }
 
 void CFILE_init_watch(void)
 {
-	if (GB_GetFunction(&read_func, PROJECT_class, "Application_Read", "", "") == 0)
+	bool has_term_func = GB_GetFunction(&_term_resize_func, PROJECT_class, "Application_Resize", "", "") == 0;
+	bool has_read_func = GB_GetFunction(&_read_func, PROJECT_class, "Application_Read", "", "") == 0;
+	
+	if (has_term_func || has_read_func)
+		OBJECT_attach((OBJECT *)CFILE_in, (OBJECT *)PROJECT_class, "Application");
+	
+	if (has_read_func)
 	{
 		//fprintf(stderr, "watch stdin\n");
-		OBJECT_attach((OBJECT *)CFILE_in, (OBJECT *)PROJECT_class, "Application");
 		CFILE_in->watch_fd = STDIN_FILENO;
 		GB_Watch(STDIN_FILENO, GB_WATCH_READ, (void *)callback_read, (intptr_t)CFILE_in);
 	}
 }
 
-
 BEGIN_METHOD_VOID(File_free)
 
 	STREAM_close(&THIS->ob.stream);
@@ -716,7 +741,7 @@ END_METHOD
 
 BEGIN_PROPERTY(Stream_Handle)
 
-	GB_ReturnInteger(STREAM_handle(CSTREAM_stream(THIS_STREAM)));
+	GB_ReturnInteger(STREAM_FD);
 
 END_PROPERTY
 
@@ -829,6 +854,12 @@ BEGIN_METHOD_VOID(Stream_Cancel)
 
 END_METHOD
 
+BEGIN_PROPERTY(Stream_IsTerm)
+
+	GB_ReturnBoolean(isatty(STREAM_FD));
+
+END_PROPERTY
+
 BEGIN_METHOD_VOID(StreamLines_next)
 
 	char *str;
@@ -844,18 +875,131 @@ BEGIN_METHOD_VOID(StreamLines_next)
 
 END_METHOD
 
+//---------------------------------------------------------------------------
+
+static void init_term_size(void *_object)
+{
+	struct winsize winSize;
+	
+	if (_term_width == 0 || _term_height == 0)
+	{
+		if (ioctl(STREAM_FD, TIOCGWINSZ, (char *)&winSize))
+			THROW_SYSTEM(errno, NULL);
+		
+		_term_width = winSize.ws_col;
+		_term_height = winSize.ws_row;
+	}
+	
+	if (!_term_init)
+	{
+		_SIGWINCH_callback = SIGNAL_register(SIGWINCH, cb_term_resize, 0);
+		_term_init = TRUE;
+	}
+}
+
+BEGIN_PROPERTY(StreamTerm_Name)
+	
+	GB_ReturnNewZeroString(ttyname(STREAM_FD));
+
+END_PROPERTY
+
+BEGIN_METHOD(StreamTerm_Resize, GB_INTEGER width; GB_INTEGER height)
+
+	struct winsize winSize = { 0 };
+	
+	winSize.ws_row = (unsigned short)Max(1, Min(65535, VARG(height)));
+  winSize.ws_col = (unsigned short)Max(1, Min(65535, VARG(width)));
+  
+	if (ioctl(STREAM_FD, TIOCSWINSZ, (char *)&winSize))
+		THROW_SYSTEM(errno, NULL);
+
+END_METHOD
+
+static void handle_term_property(void *_object, void *_param, bool iflag, int flag)
+{
+	struct termios ttmode;
+	
+	if (tcgetattr(STREAM_FD, &ttmode))
+		THROW_SYSTEM(errno, "");
+	
+	if (READ_PROPERTY)
+		GB_ReturnBoolean(((iflag ? ttmode.c_iflag : ttmode.c_lflag) & flag) == flag);
+	else
+	{
+		if (VPROP(GB_BOOLEAN))
+		{
+			if (iflag)
+				ttmode.c_iflag |= flag;
+			else
+				ttmode.c_lflag |= flag;
+		}
+		else
+		{
+			if (iflag)
+				ttmode.c_iflag &= ~flag;
+			else
+				ttmode.c_lflag &= ~flag;
+		}
+
+		if (tcsetattr(STREAM_FD, TCSANOW, &ttmode))
+		THROW_SYSTEM(errno, "");
+	}
+}
+
+BEGIN_PROPERTY(StreamTerm_Echo)
+
+	handle_term_property(_object, _param, FALSE, ECHO);
+
+END_PROPERTY
+
+BEGIN_PROPERTY(StreamTerm_FlowControl)
+
+	handle_term_property(_object, _param, TRUE, IXON | IXOFF);
+
+END_PROPERTY
+
+BEGIN_PROPERTY(StreamTerm_Width)
+
+	init_term_size(_object);
+	GB_ReturnInteger(_term_width);
+
+END_PROPERTY
+
+BEGIN_PROPERTY(StreamTerm_Height)
+
+	init_term_size(_object);
+	GB_ReturnInteger(_term_height);
+
+END_PROPERTY
+
 #endif
 
-GB_DESC NATIVE_StreamLines[] = 
+GB_DESC StreamLinesDesc[] = 
 {
-	GB_DECLARE(".Stream.Lines", 0), GB_VIRTUAL_CLASS(),
+	GB_DECLARE_VIRTUAL(".Stream.Lines"),
 	
 	GB_METHOD("_next", "s", StreamLines_next, NULL),
 	
 	GB_END_DECLARE
 };
 
-GB_DESC NATIVE_Stream[] =
+GB_DESC StreamTermDesc[] = 
+{
+	GB_DECLARE_VIRTUAL(".Stream.Term"),
+	
+	GB_PROPERTY_READ("Name", "s", StreamTerm_Name),
+	GB_PROPERTY("Echo", "b", StreamTerm_Echo),
+	GB_PROPERTY("FlowControl", "b", StreamTerm_FlowControl),
+	GB_PROPERTY_READ("Width", "i", StreamTerm_Width),
+	GB_PROPERTY_READ("W", "i", StreamTerm_Width),
+	GB_PROPERTY_READ("Height", "i", StreamTerm_Height),
+	GB_PROPERTY_READ("H", "i", StreamTerm_Height),
+	GB_METHOD("Resize", NULL, StreamTerm_Resize, "(Width)i(Height)i"),
+	
+	GB_END_DECLARE
+};
+
+GB_DESC StreamDesc[] =
 {
 	GB_DECLARE("Stream", sizeof(CSTREAM)),
 	GB_NOT_CREATABLE(),
@@ -870,8 +1014,10 @@ GB_DESC NATIVE_Stream[] =
 	GB_PROPERTY("Blocking", "b", Stream_Blocking),
 	GB_PROPERTY("Tag", "v", Stream_Tag),
 	GB_METHOD("ReadLine", "s", Stream_ReadLine, "[(Escape)s]"),
+	GB_PROPERTY_READ("IsTerm", "b", Stream_IsTerm),
 	
 	GB_PROPERTY_SELF("Lines", ".Stream.Lines"),
+	GB_PROPERTY_SELF("Term", ".Stream.Term"),
 	
 	GB_METHOD("Begin", NULL, Stream_Begin, NULL),
 	GB_METHOD("Send", NULL, Stream_End, NULL),
@@ -881,10 +1027,9 @@ GB_DESC NATIVE_Stream[] =
 };
 
 
-GB_DESC NATIVE_StatPerm[] =
+GB_DESC StatPermDesc[] =
 {
-	GB_DECLARE(".Stat.Perm", 0),
-	GB_VIRTUAL_CLASS(),
+	GB_DECLARE_VIRTUAL(".Stat.Perm"),
 
 	GB_METHOD("_get", "s", CFILE_perm_get, "(UserOrGroup)s"),
 	GB_PROPERTY_READ("User", "s", CFILE_perm_user),
@@ -895,7 +1040,7 @@ GB_DESC NATIVE_StatPerm[] =
 };
 
 
-GB_DESC NATIVE_Stat[] =
+GB_DESC StatDesc[] =
 {
 	GB_DECLARE("Stat", sizeof(CSTAT)),
 	GB_NOT_CREATABLE(),
@@ -924,7 +1069,7 @@ GB_DESC NATIVE_Stat[] =
 };
 
 
-GB_DESC NATIVE_File[] =
+GB_DESC FileDesc[] =
 {
 	GB_DECLARE("File", sizeof(CFILE)),
 	GB_INHERITS("Stream"),
@@ -956,6 +1101,7 @@ GB_DESC NATIVE_File[] =
 
 	GB_EVENT("Read", NULL, NULL, &EVENT_Read),
 	GB_EVENT("Write", NULL, NULL, &EVENT_Write),
+	GB_EVENT("Resize", NULL, NULL, &EVENT_Resize),
 
 	GB_END_DECLARE
 };
