@@ -71,7 +71,7 @@ static const SYSLOG_PREFIX _syslog_prefix[] =
 
 BEGIN_PROPERTY(User_Home)
 
-	GB_ReturnString(PROJECT_get_home());
+	GB_ReturnString(FILE_get_home());
 
 END_PROPERTY
 
@@ -181,7 +181,7 @@ BEGIN_PROPERTY(System_Backtrace)
 
 	GB_ReturnObject(DEBUG_get_string_array_from_backtrace(bt));
 	STACK_free_backtrace(&bt);
-	
+
 END_PROPERTY
 
 
@@ -197,10 +197,10 @@ BEGIN_METHOD(System_GetExternSymbol, GB_STRING library; GB_STRING name)
 	char *library = GB_ToZeroString(ARG(library));
 	char *name = GB_ToZeroString(ARG(name));
 	void *ptr = NULL;
-	
+
 	if (*library && *name)
 		ptr = EXTERN_get_symbol(library, name);
-	
+
 	GB_ReturnPointer(ptr);
 
 END_METHOD
@@ -351,7 +351,7 @@ GB_DESC NATIVE_System[] =
 
 	GB_CONSTANT("Family", "s", SYSTEM),
 	GB_CONSTANT("Architecture", "s", ARCHITECTURE),
-	
+
 	GB_STATIC_METHOD("GetExternSymbol", "p", System_GetExternSymbol, "(Library)s(Symbol)s"),
 	GB_STATIC_METHOD("_Breakpoint", NULL, System_Breakpoint, NULL),
 
