@@ -100,11 +100,12 @@ static const CLASS_INIT init_list[] =
   { NATIVE_Object, NULL },
   { NATIVE_Collection, &CLASS_Collection, CQA_COLLECTION },
   { NATIVE_Error, NULL },
-  { NATIVE_StreamLines, NULL },
-  { NATIVE_Stream, &CLASS_Stream },
-  { NATIVE_StatPerm, NULL },
-  { NATIVE_Stat, &CLASS_Stat },
-  { NATIVE_File, &CLASS_File },
+  { StreamLinesDesc, NULL },
+  { StreamTermDesc, NULL },
+  { StreamDesc, &CLASS_Stream },
+  { StatPermDesc, NULL },
+  { StatDesc, &CLASS_Stat },
+  { FileDesc, &CLASS_File },
   { NATIVE_AppEnv, &CLASS_AppEnv },
   { NATIVE_AppArgs, &CLASS_AppArgs },
   { NATIVE_App, &CLASS_Application },
