@@ -26,7 +26,7 @@
 #include <ctype.h>
 #include "gb_common.h"
 
-const char COMMON_tolower[256] = 
+const unsigned char COMMON_tolower[256] =
 	"\x00\x01\x02\x03\x04\x05\x06\x07\x08\x09\x0A\x0B\x0C\x0D\x0E\x0F"
 	"\x10\x11\x12\x13\x14\x15\x16\x17\x18\x19\x1A\x1B\x1C\x1D\x1E\x1F"
 	" !\"#$%&'()*+,-./"
@@ -44,7 +44,7 @@ const char COMMON_tolower[256] =
 	"\xE0\xE1\xE2\xE3\xE4\xE5\xE6\xE7\xE8\xE9\xEA\xEB\xEC\xED\xEE\xEF"
 	"\xF0\xF1\xF2\xF3\xF4\xF5\xF6\xF7\xF8\xF9\xFA\xFB\xFC\xFD\xFE\xFF";
 
-const char COMMON_toupper[256] =
+const unsigned char COMMON_toupper[256] =
 	"\x00\x01\x02\x03\x04\x05\x06\x07\x08\x09\x0A\x0B\x0C\x0D\x0E\x0F"
 	"\x10\x11\x12\x13\x14\x15\x16\x17\x18\x19\x1A\x1B\x1C\x1D\x1E\x1F"
 	" !\"#$%&'()*+,-./"
@@ -68,7 +68,7 @@ int COMMON_strcasecmp(const char *s1, const char *s2)
   register offset_t i;
   register int d;
   register char c;
-  
+
   for (i = 0;; i++)
   {
     c = COMMON_tolower[(unsigned char)s1[i]];
@@ -87,7 +87,7 @@ int COMMON_strncasecmp(const char *s1, const char *s2, size_t n)
   register offset_t i;
   register int d;
   register char c;
-  
+
   for (i = 0; i < n; i++)
   {
     c = COMMON_tolower[(unsigned char)s1[i]];
@@ -97,7 +97,7 @@ int COMMON_strncasecmp(const char *s1, const char *s2, size_t n)
     else if (d > 0)
       return 1;
   }
-  
+
   return 0;
 }
 
