@@ -1,23 +1,23 @@
 /***************************************************************************
 
-  gb_table.h
+	gb_table.h
 
-  (c) 2000-2013 Benoît Minisini <gambas@users.sourceforge.net>
+	(c) 2000-2013 Benoît Minisini <gambas@users.sourceforge.net>
 
-  This program is free software; you can redistribute it and/or modify
-  it under the terms of the GNU General Public License as published by
-  the Free Software Foundation; either version 2, or (at your option)
-  any later version.
+	This program is free software; you can redistribute it and/or modify
+	it under the terms of the GNU General Public License as published by
+	the Free Software Foundation; either version 2, or (at your option)
+	any later version.
 
-  This program is distributed in the hope that it will be useful,
-  but WITHOUT ANY WARRANTY; without even the implied warranty of
-  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-  GNU General Public License for more details.
+	This program is distributed in the hope that it will be useful,
+	but WITHOUT ANY WARRANTY; without even the implied warranty of
+	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+	GNU General Public License for more details.
 
-  You should have received a copy of the GNU General Public License
-  along with this program; if not, write to the Free Software
-  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
-  MA 02110-1301, USA.
+	You should have received a copy of the GNU General Public License
+	along with this program; if not, write to the Free Software
+	Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
+	MA 02110-1301, USA.
 
 ***************************************************************************/
 
@@ -27,30 +27,34 @@
 #include "gb_array.h"
 
 #define NO_SYMBOL (-1)
+//#define TABLE_USE_KEY 1
 
 typedef
-  struct {
-    char *name;
-    int len;
-    }
-  PACKED
-  SYMBOL;
+	struct {
+		char *name;
+		int len;
+#if TABLE_USE_KEY
+		uint key;
+#endif
+		}
+	PACKED
+	SYMBOL;
 
 typedef
-  enum {
-    TF_NORMAL = 0,
-    TF_IGNORE_CASE = 1
-    }
-  TABLE_FLAG;
+	enum {
+		TF_NORMAL = 0,
+		TF_IGNORE_CASE = 1
+		}
+	TABLE_FLAG;
 
 
 typedef
-  struct _table {
-    SYMBOL *symbol;
+	struct _table {
+		SYMBOL *symbol;
 		ushort *sort;
-    TABLE_FLAG flag;
-    }
-  TABLE;
+		TABLE_FLAG flag;
+		}
+	TABLE;
 
 void TABLE_create_static(TABLE *table, size_t size, TABLE_FLAG flag);
 void TABLE_delete_static(TABLE *table);
@@ -63,7 +67,6 @@ char TABLE_compare_ignore_case(const char *s1, int len1, const char *s2, int len
 char TABLE_compare(const char *s1, int len1, const char *s2, int len2);
 char TABLE_compare_ignore_case_len(const char *s1, int len1, const char *s2, int len2);
 
-//int TABLE_count(TABLE *table);
 #define TABLE_count(_table) (ARRAY_count((_table)->symbol))
 const char *TABLE_get_symbol_name(TABLE *table, int index);
 const char *TABLE_get_symbol_name_suffix(TABLE *table, int index, const char* suffix);
@@ -71,17 +74,17 @@ const char *SYMBOL_get_name(SYMBOL *sym);
 
 bool TABLE_find_symbol(TABLE *table, const char *name, int len, int *index);
 bool TABLE_add_symbol(TABLE *table, const char *name, int len, int *index);
-void TABLE_sort(TABLE *table);
 void TABLE_print(TABLE *table, bool sort);
-/*PUBLIC bool TABLE_copy_symbol(TABLE *dst, TABLE *src, int index_src, SYMBOL **symbol, int *index);*/
-void TABLE_add_new_symbol_without_sort(TABLE *table, const char *name, int len, int sort, SYMBOL **symbol, int *index);
-
-int SYMBOL_find(void *symbol, ushort *sort, int n_symbol, size_t s_symbol, int flag, const char *name, int len, const char *prefix);
-//bool SYMBOL_find_old(void *symbol, int n_symbol, size_t s_symbol, int flag, const char *name, int len, const char *prefix, int *result);
 
 #define TABLE_get_symbol(table, ind) ((SYMBOL *)ARRAY_get((table)->symbol, ind))
 
 SYMBOL *TABLE_get_symbol_sort(TABLE *table, int index);
 void TABLE_copy_symbol_with_prefix(TABLE *table, int ind_src, char prefix, int *index);
 
+int SYMBOL_find(void *symbol, ushort *sort, int n_symbol, size_t s_symbol, int flag, const char *name, int len, const char *prefix);
+#if TABLE_USE_KEY
+void SYMBOL_compute_keys(void *symbol, int n_symbol, size_t s_symbol, int flag);
+#else
+#define SYMBOL_compute_keys(_symbol, _nsymbol, _ssymbol, _flag)
+#endif
 #endif
