@@ -35,13 +35,45 @@
 
 #include "gb_table.h"
 
-
 #define SYM(table, ind) (TABLE_get_symbol(table, ind))
 #define SSYM(_symbol, _pos, _size) ((SYMBOL *)((char *)(_symbol) + (_pos) * (_size)))
 
-
 static char _buffer[MAX_SYMBOL_LEN + 1];
 
+#if TABLE_USE_KEY
+static inline uint make_key(const char *sym, int len)
+{
+	static void *jump[] = { &&__LEN_0, &&__LEN_1, &&__LEN_2, &&__LEN_3 };
+	const uchar *p = (uchar *)sym;
+	
+	if (len >= 4)
+		return (p[0] << 24) + (p[1] << 16) + (p[2] << 8) + p[3];
+	
+	goto *jump[len];
+	
+	__LEN_0: return 0;
+	__LEN_1: return (p[0] << 24);
+	__LEN_2: return (p[0] << 24) + (p[1] << 16);
+	__LEN_3: return (p[0] << 24) + (p[1] << 16) + (p[2] << 8);
+}
+
+static inline uint make_key_ignore_case(const char *sym, int len)
+{
+	static void *jump[] = { &&__LEN_0, &&__LEN_1, &&__LEN_2, &&__LEN_3 };
+	const uchar *p = (uchar *)sym;
+	
+	if (len >= 4)
+		return (tolower(p[0]) << 24) + (tolower(p[1]) << 16) + (tolower(p[2]) << 8) + tolower(p[3]);
+	
+	goto *jump[len];
+	
+	__LEN_0: return 0;
+	__LEN_1: return (tolower(p[0]) << 24);
+	__LEN_2: return (tolower(p[0]) << 24) + (tolower(p[1]) << 16);
+	__LEN_3: return (tolower(p[0]) << 24) + (tolower(p[1]) << 16) + (tolower(p[2]) << 8);
+}
+#endif
+
 char TABLE_compare(const char *s1, int len1, const char *s2, int len2)
 {
 	int i;
@@ -108,7 +140,7 @@ char TABLE_compare_ignore_case_len(const char *s1, int len1, const char *s2, int
 	return 0;
 }
 
-static bool search(void *symbol, ushort *sort, int n_symbol, size_t size, int flag, const char *name, int len, int *index)
+static inline bool search(void *symbol, ushort *sort, int n_symbol, size_t size, const char *name, int len, int *index)
 {
 	int pos, deb, fin;
 	SYMBOL *sym;
@@ -116,218 +148,150 @@ static bool search(void *symbol, ushort *sort, int n_symbol, size_t size, int fl
 	int result; // must be an integer (or a short) because uchar - uchar may not fit in a char!
 	const uchar *s1;
 	const uchar *s2;
-
+#if TABLE_USE_KEY
+	uint key = make_key(name, len);
+#endif
+	
 	pos = 0;
 	deb = 0;
 	fin = n_symbol; //ARRAY_count(table->symbol);
 
-	if (flag) // == TF_IGNORE_CASE
-	{
-		for(;;)
-		{
-			if (UNLIKELY(deb >= fin))
-			{
-				*index = deb;
-				return FALSE;
-			}
-	
-			pos = (deb + fin) >> 1;
-	
-			//sym = SYM(table, SYM(table, pos)->sort); /*&table->symbol[table->symbol[pos].sort];*/
-			
-			sym = SSYM(symbol, sort[pos], size);
-
-			if (LIKELY(len < sym->len))
-				goto __T_LOWER;
-			else if (LIKELY(len > sym->len))
-				goto __T_GREATER;
-			
-			//if (LIKELY(len > 0))
-			{
-				s1 = (uchar *)name;
-				s2 = (uchar *)sym->name;
-			
-				l = len;
-				
-				for(;;)
-				{
-					/*result = (*s1 - *s2) & ~0x20;
-					
-					if (LIKELY(result < 0))
-						goto __T_LOWER;
-					else if (LIKELY(result > 0))
-						goto __T_GREATER;*/
-					
-					result = tolower(*s1) - tolower(*s2);
-					
-					if (LIKELY(result < 0))
-						goto __T_LOWER;
-					else if (LIKELY(result > 0))
-						goto __T_GREATER;
-					
-					if (UNLIKELY(--l == 0))
-						break;
-					
-					s1++;
-					s2++;
-				}
-			
-			}  
-
-			*index = pos;
-			return TRUE;
-			
-			__T_LOWER: fin = pos; continue;
-			__T_GREATER: deb = pos + 1; continue;
-		}
-	}
-	else
-	{
-		for(;;)
-		{
-			if (UNLIKELY(deb >= fin))
-			{
-				*index = deb;
-				return FALSE;
-			}
-	
-			pos = (deb + fin) >> 1;
-	
-			sym = SSYM(symbol, sort[pos], size);
-			
-			if (LIKELY(len < sym->len))
-				goto __B_LOWER;
-			else if (LIKELY(len > sym->len))
-				goto __B_GREATER;
-			
-			//if (LIKELY(len > 0))
-			{
-				s1 = (uchar *)name;
-				s2 = (uchar *)sym->name;
-				
-				l = len;
-			
-				for(;;)
-				{
-					result = *s1 - *s2;
-					
-					if (LIKELY(result < 0))
-						goto __B_LOWER;
-					else if (LIKELY(result > 0))
-						goto __B_GREATER;
-					
-					if (UNLIKELY(--l == 0))
-						break;
-					
-					s1++;
-					s2++;
-				}
-			}
-
-			*index = pos;
-			return TRUE;
-			
-			__B_LOWER: fin = pos; continue;
-			__B_GREATER: deb = pos + 1; continue;
-		}    
-	}
-}
-
-
-int SYMBOL_find(void *symbol, ushort *sort, int n_symbol, size_t s_symbol, int flag, const char *name, int len, const char *prefix)
-{
-	int index;
-	int len_prefix;
-
-	if (UNLIKELY(prefix != NULL))
-	{
-		len_prefix = strlen(prefix);
-
-		if (UNLIKELY((len + len_prefix) > MAX_SYMBOL_LEN))
-			ERROR_panic("SYMBOL_find: prefixed symbol too long");
-
-		strcpy(_buffer, prefix);
-		strcpy(&_buffer[len_prefix], name);
-		len += len_prefix;
-		name = _buffer;
-	}
-
-	if (LIKELY(search(symbol, sort, n_symbol, s_symbol, flag, name, len, &index)))
-		return sort[index];
-	else
-		return NO_SYMBOL;
-}
-
-#if 0
-bool SYMBOL_find_old(void *symbol, int n_symbol, size_t s_symbol, int flag,
-															const char *name, int len, const char *prefix, int *result)
-{
-	int pos, deb, fin;
-	int cmp;
-	int (*cmp_func)(const char *, int, const char *, int);
-	SYMBOL *sym;
-	int index;
-	int len_prefix;
-
-	cmp_func = ((flag == TF_IGNORE_CASE) ? TABLE_compare_ignore_case : TABLE_compare);
-
-	pos = 0;
-	deb = 0;
-	fin = n_symbol;
-
-	if (UNLIKELY(prefix != NULL))
-	{
-		len_prefix = strlen(prefix);
-
-		if (UNLIKELY((len + len_prefix) > MAX_SYMBOL_LEN))
-			ERROR_panic("SYMBOL_find: prefixed symbol too long");
-
-		strcpy(_buffer, prefix);
-		strcpy(&_buffer[len_prefix], name);
-		len += len_prefix;
-		name = _buffer;
-	}
-
 	for(;;)
 	{
 		if (UNLIKELY(deb >= fin))
 		{
-			*result = NO_SYMBOL;
+			*index = deb;
 			return FALSE;
 		}
 
 		pos = (deb + fin) >> 1;
 
-		index = ((SYMBOL *)((char *)symbol + s_symbol * pos))->sort;
-		sym = (SYMBOL *)((char *)symbol + s_symbol * index);
-
-		cmp = (*cmp_func)(name, len, sym->name, sym->len);
-
-		if (UNLIKELY(cmp == 0))
+		sym = SSYM(symbol, sort[pos], size);
+		
+		if (LIKELY(len < sym->len))
+			goto __B_LOWER;
+		else if (LIKELY(len > sym->len))
+			goto __B_GREATER;
+		
+#if TABLE_USE_KEY
+		if (key < sym->key) goto __B_LOWER;
+		if (key > sym->key) goto __B_GREATER;
+		
+		l = len - 4;
+		
+		if (l > 0)
 		{
-			*result = index;
-			return TRUE;
+			s1 = (uchar *)name + 4;
+			s2 = (uchar *)sym->name + 4;
+			
+#else
+		{
+			l = len;
+			s1 = (uchar *)name;
+			s2 = (uchar *)sym->name;
+#endif
+			for(;;)
+			{
+				result = *s1 - *s2;
+				
+				if (LIKELY(result < 0))
+					goto __B_LOWER;
+				else if (LIKELY(result > 0))
+					goto __B_GREATER;
+				
+				if (UNLIKELY(--l == 0))
+					break;
+				
+				s1++;
+				s2++;
+			}
 		}
 
-		if (cmp < 0)
-			fin = pos;
-		else
-			deb = pos + 1;
-	}
+		*index = pos;
+		return TRUE;
+		
+		__B_LOWER: fin = pos; continue;
+		__B_GREATER: deb = pos + 1; continue;
+	}    
 }
+
+
+static inline bool search_ignore_case(void *symbol, ushort *sort, int n_symbol, size_t size, const char *name, int len, int *index)
+{
+	int pos, deb, fin;
+	SYMBOL *sym;
+	int l;
+	int result; // must be an integer (or a short) because uchar - uchar may not fit in a char!
+	const uchar *s1;
+	const uchar *s2;
+#if TABLE_USE_KEY
+	uint key = make_key_ignore_case(name, len);
 #endif
 
-const char *SYMBOL_get_name(SYMBOL *sym)
-{
-	int len;
+	pos = 0;
+	deb = 0;
+	fin = n_symbol;
 
-	len = Min(MAX_SYMBOL_LEN, sym->len);
-	memcpy(_buffer, sym->name, len);
-	_buffer[len] = 0;
+	for(;;)
+	{
+		if (UNLIKELY(deb >= fin))
+		{
+			*index = deb;
+			return FALSE;
+		}
 
-	return _buffer;
+		pos = (deb + fin) >> 1;
+
+		sym = SSYM(symbol, sort[pos], size);
+
+		if (LIKELY(len < sym->len))
+			goto __T_LOWER;
+		else if (LIKELY(len > sym->len))
+			goto __T_GREATER;
+
+#if TABLE_USE_KEY
+		if (key < sym->key) goto __T_LOWER;
+		if (key > sym->key) goto __T_GREATER;
+		
+		l = len - 4;
+		
+		if (l > 0)
+		{
+			s1 = (uchar *)name + 4;
+			s2 = (uchar *)sym->name + 4;
+#else
+		{
+			l = len;
+			s1 = (uchar *)name;
+			s2 = (uchar *)sym->name;
+#endif
+		
+			for(;;)
+			{
+				result = tolower(*s1) - tolower(*s2);
+				
+				if (LIKELY(result < 0))
+					goto __T_LOWER;
+				else if (LIKELY(result > 0))
+					goto __T_GREATER;
+				
+				if (UNLIKELY(--l == 0))
+					break;
+				
+				s1++;
+				s2++;
+			}
+		}
+
+		*index = pos;
+		return TRUE;
+		
+		__T_LOWER: fin = pos; continue;
+		__T_GREATER: deb = pos + 1; continue;
+	}
 }
 
+
 const char *TABLE_get_symbol_name(TABLE *table, int index)
 {
 	if (UNLIKELY((index < 0) || (index >= ARRAY_count(table->symbol))))
@@ -421,7 +385,10 @@ bool TABLE_find_symbol(TABLE *table, const char *name, int len, int *index)
 	count = ARRAY_count(tsym);
 	size = ARRAY_size(tsym);
 
-	result = search(tsym, table->sort, count, size, table->flag, name, len, &ind);
+	if (table->flag)
+		result = search_ignore_case(tsym, table->sort, count, size, name, len, &ind);
+	else
+		result = search(tsym, table->sort, count, size, name, len, &ind);
 	
 	if (result)
 		*index = table->sort[ind];
@@ -429,7 +396,7 @@ bool TABLE_find_symbol(TABLE *table, const char *name, int len, int *index)
 	return result;
 }
 
-
+#if 0
 void TABLE_add_new_symbol_without_sort(TABLE *table, const char *name, int len, int sort, SYMBOL **symbol, int *index)
 {
 	SYMBOL *sym;
@@ -449,7 +416,7 @@ void TABLE_add_new_symbol_without_sort(TABLE *table, const char *name, int len,
 	if (symbol) *symbol = sym; /*&table->symbol[ind];*/
 	if (index) *index = count;
 }
-
+#endif
 
 bool TABLE_add_symbol(TABLE *table, const char *name, int len, int *index)
 {
@@ -465,7 +432,10 @@ bool TABLE_add_symbol(TABLE *table, const char *name, int len, int *index)
 	count = ARRAY_count(table->symbol);
 	size = ARRAY_size(table->symbol);
 	
-	result = search(table->symbol, table->sort, count, size, table->flag, name, len, &ind);
+	if (table->flag)
+		result = search_ignore_case(table->symbol, table->sort, count, size, name, len, &ind);
+	else
+		result = search(table->symbol, table->sort, count, size, name, len, &ind);
 
 	if (!result)
 	{
@@ -473,6 +443,9 @@ bool TABLE_add_symbol(TABLE *table, const char *name, int len, int *index)
 
 		sym->name = (char *)name;
 		sym->len = len;
+#if TABLE_USE_KEY
+		sym->key = table->flag ? make_key_ignore_case(name, len) : make_key(name, len);
+#endif
 		
 		/*
 		printf("TABLE_add_symbol: %.*s %d %d\n", len, name, ((CLASS_SYMBOL *)sym)->global.type,
@@ -484,19 +457,6 @@ bool TABLE_add_symbol(TABLE *table, const char *name, int len, int *index)
 		if (count > ind)
 			memmove(&table->sort[ind + 1], &table->sort[ind], sizeof(ushort) * (count - ind));
 
-		/*s1 = SSYM(table->symbol, count, size);
-		s2 = (SYMBOL *)((char *)s1 - size);
-		
-		i = count - ind;
-		while (i)
-		{
-			s1->sort = s2->sort;
-			i--;
-			s1 = (SYMBOL *)((char *)s1 - size);
-			s2 = (SYMBOL *)((char *)s2 - size);
-		}*/
-
-		//SYM(table, ind)->sort = (ushort)count;
 		table->sort[ind] = (ushort)count;
 		ind = count;
 	}
@@ -571,3 +531,73 @@ SYMBOL *TABLE_get_symbol_sort(TABLE *table, int index)
 	return TABLE_get_symbol(table, table->sort[index]);
 }
 
+// Symbol tables with no TABLE structure
+
+int SYMBOL_find(void *symbol, ushort *sort, int n_symbol, size_t s_symbol, int flag, const char *name, int len, const char *prefix)
+{
+	int index;
+	int len_prefix;
+
+	if (UNLIKELY(prefix != NULL))
+	{
+		len_prefix = strlen(prefix);
+
+		if (UNLIKELY((len + len_prefix) > MAX_SYMBOL_LEN))
+			ERROR_panic("SYMBOL_find: prefixed symbol too long");
+
+		strcpy(_buffer, prefix);
+		strcpy(&_buffer[len_prefix], name);
+		len += len_prefix;
+		name = _buffer;
+	}
+
+	if (flag)
+	{
+		if (search_ignore_case(symbol, sort, n_symbol, s_symbol, name, len, &index))
+			return sort[index];
+	}
+	else
+	{
+		if (search(symbol, sort, n_symbol, s_symbol, name, len, &index))
+			return sort[index];
+	}
+
+	return NO_SYMBOL;
+}
+
+
+const char *SYMBOL_get_name(SYMBOL *sym)
+{
+	int len;
+
+	len = Min(MAX_SYMBOL_LEN, sym->len);
+	memcpy(_buffer, sym->name, len);
+	_buffer[len] = 0;
+
+	return _buffer;
+}
+
+#if TABLE_USE_KEY
+void SYMBOL_compute_keys(void *symbol, int n_symbol, size_t s_symbol, int flag)
+{
+	int i;
+	SYMBOL *sym;
+	
+	if (flag)
+	{
+		for (i = 0; i < n_symbol; i++)
+		{
+			sym = SSYM(symbol, i, s_symbol);
+			sym->key = make_key_ignore_case(sym->name, sym->len);
+		}
+	}
+	else
+	{
+		for (i = 0; i < n_symbol; i++)
+		{
+			sym = SSYM(symbol, i, s_symbol);
+			sym->key = make_key(sym->name, sym->len);
+		}
+	}
+}
+#endif
